@@ -1,6 +1,6 @@
 package Gtk2::Ex::HyperTextBuffer;
 
-our $VERSION = '0.10';
+our $VERSION = '0.12';
 
 use strict;
 use Gtk2;
@@ -29,10 +29,12 @@ instead of plain text. It also keeps an undo-/redo-stack.
 
 # It seems to me that 10 * PANGO_SCALE is the normal font size
 # TODO make tag styles configable
+# FIXME replace "weight" with "scale" ?
 our %TAGS = (
 	bold      => [weight => PANGO_WEIGHT_BOLD],
 	italic    => [style => 'italic'],
 	underline => [underline => 'single'],
+	strike    => [strikethrough => 'true', foreground => 'grey'],
 	verbatim  => [family => 'monospace'], # wrap_mode => 'none'], # Bug withwrap_mode !?
 	Verbatim  => [family => 'monospace', wrap_mode => 'none', indent => 40],
 	link      => [foreground => 'blue', underline => 'single'],
@@ -297,6 +299,7 @@ sub get_parse_tree {
 	}
 
 	# Check lines around headings - FIXME very ugly implementation
+	# Also add sanity check to merge blocks of 'verbatim' to 'Verbatim'
 	for (0 .. $#$root) {
 		next unless ref $$root[$_] and $$root[$_][0] =~ /^head/;
 		$$root[$_][-1] =~ s/(\n*)$//;
@@ -363,22 +366,115 @@ sub create_link_tag {
 	return $tag;
 }
 
-=item C<select_word()>
+=item C<auto_selection(TAG, ITER)>
 
-TODO
+Returns start and end iterators for the current selection.
+If there is no selection in the buffer TAG and ITER are used to
+automaticly select a range if possible.
+ITER is optional and defaults to the cursor.
+
+=cut
+
+sub auto_selection {
+	my ($self, $tag, $iter) = @_;
+	my ($start, $end) = $self->get_selection_bounds;
+	#warn "selection: $start, $end tag: $tag\n";
+	if (!$end or $start == $end) { # autoselect word if defined tag
+		if ($tag eq 'link') {
+			($start, $end) = $self->select_link($iter);
+			($start, $end) = $self->select_word($iter)
+				if !$end or $start == $end;
+		}
+		elsif ($tag =~ /^head/) {
+			($start, $end) = $self->select_line($iter);
+		}
+		elsif ($tag and $tag !~ /^[A-Z]/) {
+			($start, $end) = $self->select_word($iter);
+		}
+		#warn "   => $start, $end\n";
+		return undef if !$end or $start == $end;
+	}
+	return ($start, $end);
+}
+
+=item C<select_word(ITER)>
+
+Selects the current word if ITER is inside a word.
+Cursor is used when ITER is undefined.
+Returns Gtk2::TextIter for start and end or undef.
 
 =cut
 
 sub select_word {
+	my ($self, $iter) = @_;
+	$iter ||= $self->get_iter_at_mark( $self->get_insert );
+	#warn "select-word: ", $iter->inside_word, "\n";
+	return unless $iter->inside_word
+		or $iter->starts_word or $iter->ends_word;
+	my $begin = $iter->copy;
+	$begin->backward_word_start unless $begin->starts_word;
+	$iter->forward_word_end unless $iter->ends_word;
+	$self->select_range($begin, $iter);
+	return ($begin, $iter);
 }
 
-=item C<select_line>
+=item C<select_line(ITER)>
 
-TODO
+Selects the line that contains ITER.
+Cursor is used when ITER is undefined.
+Returns Gtk2::TextIter for start and end or undef.
 
 =cut
 
 sub select_line {
+	my ($self, $iter) = @_;
+	$iter ||= $self->get_iter_at_mark( $self->get_insert );
+	my $begin = $iter->copy;
+	$begin = $self->get_iter_at_line( $begin->get_line )
+		unless $begin->starts_line;
+	while (! $iter->ends_line) { $iter->forward_char }
+	$self->select_range($begin, $iter);
+	return ($begin, $iter);
+}
+
+=item C<select_tag(ITER, TAG)>
+
+Selects TAG if ITER has TAG.
+Cursor is used when ITER is undefined.
+Tag can be either a Gtk2::TextTag or a tag name.
+Returns Gtk2::TextIter for start and end or undef.
+
+=cut
+
+sub select_tag {
+	my ($self, $iter, $tag) = @_;
+	#warn "select_tag: $tag\n";
+	$tag = $self->get_tag_table->lookup($tag) unless ref $tag;
+	$iter ||= $self->get_iter_at_mark( $self->get_insert );
+	return unless $iter->has_tag($tag) or $iter->toggles_tag($tag);
+	my $begin = $iter->copy;
+	$begin->backward_to_tag_toggle($tag) unless $begin->begins_tag($tag);
+	$iter->forward_to_tag_toggle($tag) unless $begin->ends_tag($tag);
+	$self->select_range($begin, $iter);
+	return ($begin, $iter);
+}
+
+=item C<select_link(ITER)>
+
+Selects link if ITER is on a link.
+Cursor is used when ITER is undefined.
+Returns Gtk2::TextIter for start and end or undef.
+
+=cut
+
+sub select_link {
+	#warn "select_link\n";
+	my ($self, $iter) = @_;
+	$iter ||= $self->get_iter_at_mark( $self->get_insert );
+	for ($iter->get_tags) {
+		next unless $_->{is_link};
+		return $self->select_tag($iter, $_);
+	}
 }
 
 =item C<apply_tag_to_selection>
@@ -392,13 +488,28 @@ sub apply_tag_to_selection {
 	
 }
 
-=item C<replace_selection>
+=item C<replace_selection(TEXT)>
 
-TODO
+Replace any selected text with TEXT.
+Returns iterators for the start and end of the newly inserted selection.
 
 =cut
 
 sub replace_selection {
+	my ($self, $text) = @_;
+	my ($start, $end) = $self->get_selection_bounds;
+	return if !$end or $start == $end;
+
+	my $_start = $start->get_offset;
+	my $_end   = $start->get_offset + length $text;
+
+	$self->delete($start, $end);
+	$self->insert($start, $text);
+
+	($start, $end) = map $self->get_iter_at_offset($_), ($_start, $_end);
+	$self->select_range($start, $end);
+
+	return $start, $end;
 }
 
 =item C<set_edit_mode_tags>
