@@ -4,17 +4,18 @@ use strict;
 use vars qw/$AUTOLOAD %Config/;
 use Gtk2;
 use Gtk2::Gdk::Keysyms;
-use POSIX qw(strftime);
+use Gtk2::SimpleList;
 use Zim::History;
 use Zim::Components::TreeView;
 use Zim::Components::PathBar;
 use Zim::Components::PageView;
+#use Zim::Components::SearchPane;
 
 # TODO make components configable/autoloading like plugins
 
-our $VERSION = '0.10';
+our $VERSION = '0.12';
 our $LONG_VERSION = << "EOT";
-zim $VERSION - A desktop wiki
+zim $VERSION - A desktop wiki and outliner
 
 Copyright (c) 2005 Jaap G Karssenberg. All rights reserved.
 This program is free software; you can redistribute it and/or
@@ -80,21 +81,26 @@ our %CONFIG = (  # Default config values
 	#icon_size	=> 'large-toolbar',
 	width		=> 500,
 	height		=> 350,
+	x		=> undef,
+	y		=> undef,
 	default_root	=> undef,
 	default_home	=> 'Home',
 	browser		=> undef,
 	file_browser	=> undef,
 	date_string	=> '%A %d/%m/%Y',
-	date_page	=> ':Date:%Y_%m_%d',
 	hist_max	=> 20,
 	recent_show	=> 7,
 	undo_max	=> 50,
 	follow_new_link => 1,
+	backsp_unindent => 1,
 	use_xdg_cache	=> 0,
 	use_tray_icon	=> 0,
 	use_gtkspell	=> 0,
 	show_spell      => 1,
-	backsp_unindent => 1,
+	spell_language	=> undef,
+	use_calendar	=> 0,
+	cal_namespace	=> ':Date:',
+	show_cal	=> 0,
 );
 
 my @ui_actions = (
@@ -109,7 +115,9 @@ my @ui_actions = (
 	[ 'HelpMenu',    undef,     '_Help'         ],
 	[ 'PathBarMenu', undef,     'P_athbar type' ],
 	# name,       stock id,         label,          accelerator,   tooltip
+	[ 'OpenRep',  'gtk-open',       '_Open...',     '<ctrl>O',     'Open',         \&on_OpenRep  ],
 	[ 'Save',     'gtk-save',       '_Save',        '<ctrl>S',     'Save page',    \&on_Save     ],
+	[ 'Rename',   undef,            '_Rename page', 'F2',          'Rename page',  \&on_Rename   ],
 	[ 'Close',    'gtk-close',      '_Close',       '<ctrl>W',     'Close window', \&on_Close    ],
 	[ 'Quit',     'gtk-quit',       '_Quit',        '<ctrl>Q',     'Quit',         \&on_Quit     ],
 	[ 'Reload',   'gtk-refresh',    '_Reload',      '<ctrl>R',     'Reload',       \&on_Reload   ],
@@ -120,7 +128,6 @@ my @ui_actions = (
 	[ 'RForward', undef,            '_Forward',     '<alt>Page_Up',   undef,       \&on_RForward ],
 	[ 'Parent',   'gtk-go-up',      '_Parent',      '<alt>Up',     'Go to parent', \&on_Parent   ],
 	[ 'Child',    'gtk-go-down',    '_Child',       '<alt>Down',   'Go to child',  \&on_Child    ],
-	[ 'Today',    undef,            'To_day',       '<alt>D',      'Today',        \&on_Today    ],
 	[ 'Home',     'gtk-home',       '_Home',        '<alt>Home',   'Go home',      \&on_Home     ],
 	[ 'JumpTo',   'gtk-jump-to',    '_Jump to...',  '<ctrl>J',     'Jump to page', \&on_JumpTo   ],
 	[ 'Help',     'gtk-help',       '_Help',        'F1',          'Help',         \&on_Help     ],
@@ -144,8 +151,14 @@ my %pathbar_map = ( 'recent' => 0, 'history' => 1, 'namespace' => 2, 'hidden' =>
 my $ui_layout = q{<ui>
 	<menubar name='MenuBar'>
 		<menu action='FileMenu'>
+			<menuitem action='OpenRep'/>
+			<separator/>
 			<menuitem action='Save'/>
 			<separator/>
+			<placeholder name='FileMods'>
+				<menuitem action='Rename'/>
+			</placeholder>
+			<separator/>
 			<menuitem action='Close'/>
 			<menuitem action='Quit'/>
 		</menu>
@@ -161,6 +174,9 @@ my $ui_layout = q{<ui>
 				<menuitem action='PBHidden'/>
 			</menu>
 			<separator/>
+			<placeholder name='PluginItems'>
+			</placeholder>
+			<separator/>
 			<menuitem action='Reload'/>
 		</menu>
 		<menu action='InsertMenu'></menu>
@@ -174,8 +190,10 @@ my $ui_layout = q{<ui>
 			<menuitem action='Forward'/>
 			<menuitem action='Parent'/>
 			<separator/>
+			<placeholder name='PluginItems'>
+			</placeholder>
+			<separator/>
 			<menuitem action='Home'/>
-			<menuitem action='Today'/>
 			<menuitem action='JumpTo'/>
 		</menu>
 		<menu action='HelpMenu'>
@@ -259,6 +277,7 @@ This method initializes all GUI objects that make up the application.
 sub gui_init {
 	my $self = shift;
 
+	## Setup the window
 	my $window = Gtk2::Window->new('toplevel');
 	$window->set_default_size(@{$self->{settings}}{'width', 'height'});
 	$window->signal_connect_swapped(delete_event => \&on_delete_event, $self);
@@ -268,19 +287,36 @@ sub gui_init {
 	$window->set_title('Zim');
 	$self->{window} = $window;
 
+	#$window->signal_connect(hide => sub { # FIXME does not work - patched in TrayIcon
+	#		@{$self->{settings}}{'x', 'y'} = $window->get_position;
+	#		warn "hiding window: @{$self->{settings}}{'x', 'y'}\n";
+	#	} );
+	$window->signal_connect(show => sub {
+			my ($x, $y) = @{$self->{settings}}{'x','y'};
+	#		warn "showing window: $x, $y\n";
+			$window->move($x,$y) if defined($x) and defined ($y);
+		} );
+			
+
 	my $vbox = Gtk2::VBox->new(0, 0);
 	$window->add($vbox);
 	$self->{vbox} = $vbox;
 
+	## Setup actions and ui for menubar and toolbar
 	my $actions = Gtk2::ActionGroup->new("Actions");
 	$actions->add_actions(\@ui_actions, $self);
 	$actions->add_toggle_actions(\@ui_toggle_actions, $self);
 	$actions->add_radio_actions(\@ui_radio_actions,
 		$pathbar_map{$self->{settings}{pathbar_type}}, \&on_TPathBar, $self);
-	$actions->get_action('Save')->set_sensitive(0) if $self->{settings}{read_only};
-	$actions->get_action('Back')->set_sensitive(0);
-	$actions->get_action('Forward')->set_sensitive(0);
-	$actions->get_action('Parent')->set_sensitive(0);
+	#$actions->get_action('Save')->set_sensitive(0) if $self->{settings}{read_only};
+	#$actions->get_action('Back')->set_sensitive(0);
+	#$actions->get_action('Forward')->set_sensitive(0);
+	#$actions->get_action('Parent')->set_sensitive(0);
+	_gtk_action_set_sensitive($actions->get_action('Save'), 0)
+		if $self->{settings}{read_only};
+	_gtk_action_set_sensitive($actions->get_action('Back'), 0);
+	_gtk_action_set_sensitive($actions->get_action('Forward'), 0);
+	_gtk_action_set_sensitive($actions->get_action('Parent'), 0);
 	$self->{actions} = $actions;
 	
 	my $ui = Gtk2::UIManager->new;
@@ -295,11 +331,17 @@ sub gui_init {
 	$self->{toolbar}->set_no_show_all(1);
 	$self->{ui} = $ui;
 
+	## General window layout
 	my $hpaned = Gtk2::HPaned->new();
 	$hpaned->set_position($self->{settings}{pane_pos});
 	$vbox->pack_start($hpaned, 1,1,0);
 	$self->{hpaned} = $hpaned;
 
+	my $l_vbox = Gtk2::VBox->new(0, 0);
+	$hpaned->add1($l_vbox);
+	$l_vbox->set_no_show_all(1);
+	$self->{l_vbox} = $l_vbox;
+
 	my $r_vbox = Gtk2::VBox->new(0, 0);
 	$hpaned->add2($r_vbox);
 	$self->{r_vbox} = $r_vbox;
@@ -309,13 +351,45 @@ sub gui_init {
 	$vbox->pack_start($statusbar, 0,1,0);
 	$self->{statusbar} = $statusbar;
 	
+	## Build the content of the side pane
+#	my $l_top_hbox = Gtk2::HBox->new(0, 0);
+#	$l_vbox->pack_start($l_top_hbox, 0,1,0);
+	
+#	my $pane_combo = Gtk2::ComboBox->new_text;
+#	$pane_combo->signal_connect(changed =>
+#		sub { $self->show_in_pane($pane_combo->get_active_text()) } );
+#	# FIXME get_active_text is since 2.6
+#	$l_top_hbox->add($pane_combo);
+#	$self->{pane_combo} = $pane_combo;
+
+	my $close_button = Gtk2::Button->new;
+	$close_button->set_relief('none');
+	$close_button->add(Gtk2::Alignment->new(0.5, 0.5, 0, 0));
+	$close_button->child->add(
+		Gtk2::Image->new_from_stock('gtk-close', 'small-toolbar') );
+	$close_button->signal_connect(clicked =>
+		sub { $actions->get_action('TPane')->set_active(0) } );
+#	$l_top_hbox->pack_end($close_button, 0,1,0);
+	
+	my $notebook = Gtk2::Notebook->new;
+	$notebook->set_show_tabs(0);
+	$notebook->set_show_border(0);
+	$l_vbox->add($notebook);
+	$self->{notebook} = $notebook;
+	
 	my $tree_view = Zim::Components::TreeView->new(app => $self);
 	$tree_view->signal_connect(row_activated =>
 		sub { $self->toggle_pane(0) if $self->{_pane_visible} == -1 } );
-	$tree_view->widget->set_no_show_all(1);
-	$hpaned->add1($tree_view->widget);
+	$self->add_to_pane($tree_view->widget, 'Index');
 	$self->{objects}{TreeView} = $tree_view;
 
+	#my $search_pane = Zim::Components::SearchPane->new(app => $self);
+	#$self->add_to_pane($search_pane->widget, 'Search');
+	#$self->{objects}{SearchPane} = $search_pane;
+	
+#	$pane_combo->set_active(0); # FIXME should be setting
+	
+	## Build content of the main area
 	my $path_bar = Zim::Components::PathBar->new(app => $self);
 	$r_vbox->pack_start($path_bar->widget, 0,1,5);
 	$self->{objects}{PathBar} = $path_bar;
@@ -340,36 +414,34 @@ sub gui_init {
 		$self->{settings}{hist_max} );
 	$self->{objects}{History} = $history;
 
-	# Some wiring
+	## Some wiring
 	$ui->get_accel_group->connect( # Ctrl-Space
-			ord(' '), ['control-mask'], ['visible'],
-			sub {
-				if ($self->{settings}{pane_vis}) {
+		ord(' '), ['control-mask'], ['visible'],
+		sub {
+			if ($self->{settings}{pane_vis}) {
 				if ($tree_view->has_focus) { $page_view->grab_focus }
 				else                       { $tree_view->grab_focus }
 			}
-			else { $self->toggle_pane($self->{_pane_visible} ? 0 : -1) }
+			else {
+				my $vis = $self->{_pane_visible} ? 0 : -1;
+				$self->toggle_pane($vis);
+				$vis ? $tree_view->grab_focus : $page_view->grab_focus ;
+			}
 		} );
 	
 	$self->signal_connect('page_loaded', sub {
 		my $self = shift;
 		my $state = $self->History->get_state;
-		$self->{actions}->get_action('Back')->set_sensitive($$state{back} ? 1:0);
-		$self->{actions}->get_action('Forward')->set_sensitive($$state{forw} ? 1:0);
+		#$self->{actions}->get_action('Back')->set_sensitive($$state{back} ? 1:0);
+		#$self->{actions}->get_action('Forward')->set_sensitive($$state{forw} ? 1:0);
+		_gtk_action_set_sensitive($self->{actions}->get_action('Back'), $$state{back} ? 1:0);
+		_gtk_action_set_sensitive($self->{actions}->get_action('Forward'), $$state{forw} ? 1:0);
 		my $namespace = ($self->{page}->namespace =~ /[^:]/) ? 1 : 0;
-		$self->{actions}->get_action('Parent')->set_sensitive($namespace);
+		#$self->{actions}->get_action('Parent')->set_sensitive($namespace);
+		_gtk_action_set_sensitive($self->{actions}->get_action('Parent'), $namespace);
 	} );
 
-	# set save interval
-	$self->{_save_timeout} = 
-		Glib::Timeout->add(5000, sub {$self->save_page; 1})
-		unless $self->{settings}{read_only};
-	
-	$window->show_all;
-	$statusbar->set_no_show_all(0);
-	$tree_view->widget->set_no_show_all(0);
-	$self->{toolbar}->set_no_show_all(0);
-	
+	## Some Optional features
 	if ($self->{settings}{use_tray_icon}) {
 		eval {
 			eval 'use Zim::Components::TrayIcon'; die $@ if $@;
@@ -390,36 +462,54 @@ sub gui_init {
 			"Failed to load spell checking,\n".
 			"do you have Gtk2::Spell installed ?", $@) if $@;
 	}
+	if ($self->{settings}{use_calendar}) {
+		eval {
+			eval 'use Zim::Components::Calendar'; die $@ if $@;
+			my $cal = Zim::Components::Calendar->new(app => $self);
+			$self->{objects}{Calendar} = $cal;
+		} ;
+		$self->error_dialog("Failed to load calendar", $@) if $@;
+	}
 
-#	unless ($self->{settings}{read_only}) {
-		# FIXME this is a ugly and quick hack
-		# FIXME should this be in the repository api ??
-#		my $root = $self->{repository}{root};
-#		my $lock_file = File::Spec->catfile($root, '.zim.pid');
-#		exit 1 if -e $lock_file and
-#			! $self->repository_lock_dialog($root);
-#		open LOCK, ">$lock_file" or $self->exit_error(
-#			"Could not write lock file:\n$lock_file" );
-#		print LOCK "$$\n";
-#		close LOCK;
-#		$self->{lock_file} = $lock_file;
-#	}
+	## Show all widgets .. well most of them
+	$window->show_all;
+	$statusbar->set_no_show_all(0);
+	$l_vbox->set_no_show_all(0);
+	$self->{toolbar}->set_no_show_all(0);
+#	my ($x, $y) = @{$self->{settings}}{'x','y'};
+#	$window->move($x,$y) if defined($x) and defined ($y);
 	
-	# Try saving the program on system signals
-	$SIG{$_} = sub {
-		print "Signal $_ caught\n";
-		$self->quit('FORCE');
-		exit 1;
-	} for qw/TERM HUP PIPE/;
+	## Try saving the program on system signals
+	for my $sig (qw/TERM HUP PIPE/) {
+		$SIG{$sig} = sub {
+			warn "Signal $sig caught\n";
+			$self->quit('FORCE');
+			exit 1;
+		};
+	}
 
 	# Try saving on desktop exit
-	$window->get_display->signal_connect( closed => sub {
-			print "Display closed\n";
-		} );
+#	$window->get_display->signal_connect( closed => sub {
+#			print "Display closed\n";
+#		} );
+}
+
+sub _gtk_action_set_sensitive { # **sigh**
+	my ($action, $bit) = @_;
+	if (Gtk2->CHECK_VERSION(2, 6, 0)) { $action->set_sensitive($bit) }
+	else { $_->set_sensitive($bit) for $action->get_proxies }
+}
+
+sub on_OpenRep {
+	my $self = pop;
+	my $rep = $self->prompt_repository_dialog;
+	$self->exec_new_window($rep) if defined $rep;
 }
 
 sub on_Save { pop->save_page('FORCE') }
 
+sub on_Rename { pop->rename_page() }
+
 sub on_Close { pop->quit() }
 
 sub on_Quit { pop->quit('FORCE') }
@@ -450,7 +540,7 @@ sub on_TStatusBar {
 
 sub on_TPane {
 	my $self = pop;
-	my $vis = $self->{objects}{TreeView}->widget->visible ? 0 : 1;
+	my $vis = $self->{l_vbox}->visible ? 0 : 1;
 	$self->toggle_pane($vis)
 }
 
@@ -502,13 +592,6 @@ sub on_Home {
 	$self->load_page($home);
 }
 
-sub on_Today {
-	my $self = pop;
-	my $page = strftime($self->{settings}{date_page}, localtime);
-	$page = $self->{repository}->resolve_page($page);
-	$self->load_page($page);
-}
-
 sub on_JumpTo { pop->jump_page_dialog }
 
 sub on_Help { pop->show_help }
@@ -621,11 +704,11 @@ This method runs the GUI application.
 sub main_loop {
 	my $self = shift;
 	
+	## Find a page to load
 	unless ($self->{page}) {
 		my $page = $self->{objects}{History}->get_current
 			|| $self->{settings}{home};
-		$page = $self->{repository}->resolve_page($page)
-			unless ref $page;
+		$page = $self->{repository}->resolve_page($page) unless ref $page;
 		$self->load_page($page);
 		# FIXME what if we are read_only and page does not exist ?
 		# 	load_apge returns undef
@@ -633,6 +716,7 @@ sub main_loop {
 		# 	need page that always exists ... like about or version page
 	}
 	
+	## Toggle some widgets
 	$self->{message_lock} = 1;
 	$self->{objects}{TreeView}->load_index();
 	my ($actions, $settings) = ($self->{actions}, $self->{settings});
@@ -644,6 +728,12 @@ sub main_loop {
 
 	$self->{objects}{PageView}->grab_focus;
 
+	## set save interval
+	$self->{_save_timeout} = 
+		Glib::Timeout->add(5000, sub {$self->save_page; 1})
+		unless $self->{settings}{read_only};
+	
+	$self->signal_emit('main');
 	Gtk2->main;
 }
 
@@ -665,24 +755,28 @@ sub on_delete_event {
 	my $self = shift;
 	my $force = uc(shift) eq 'FORCE';
 
+	eval {
+		my ($settings, $window) = @{$self}{'settings', 'window'};
+		@{$settings}{'width', 'height'} = $window->get_size;
+		@{$settings}{'x', 'y'} = $window->get_position if $window->visible;
+			# position is 0,0 when window is hidden
+	};
+	
 	if ($self->{settings}{hide_on_delete} and !$force) {
 		$self->{window}->hide;
 		return 1;
 	}
 	
-	#print STDERR "Zim saving state\n";
-	
-	eval {	@{$self->{settings}}{'width', 'height'} =
-			$self->{window}->get_size          };
-	
 	$self->{save_lock} = 0; # just to be sure
 	$self->save_page() || return 1;
 	$self->save_config;
 
 	unless ($self->{settings}{read_only}) {
-		my ($vol, $dirs) = File::Spec->splitpath($self->{settings}{hist_file});
-		_mkdir($vol, $dirs);
-		$self->{objects}{History}->write;
+		my ($vol, $dirs, undef) =
+			File::Spec->splitpath($self->{settings}{hist_file});
+		my $dir = File::Spec->catpath($vol, $dirs);
+		$self->{objects}{History}->write if -d $dir;
+			# Only safe history if directory exists ..
 	}
 
 	if ($self->{lock_file}) {
@@ -704,6 +798,7 @@ Page names are resolved as relative links first.
 sub link_clicked {
 	my ($self, $link) = @_;
 	$link = $$link[1] if ref $link; # link_data = [bit, link]
+	#warn "link clicked: >>$link<<\n";
 	return warn "Warning: You tried to folow an empty link.\n"
 		unless length $link;
 
@@ -780,57 +875,44 @@ use C<link_clicked()> for relative page names, urls etc.
 =cut
 
 sub load_page {
-	my ($self, $page, @args) = @_;
+	my ($self, $page) = @_;
 	
 	if ($self->{page}) {
 		$self->{save_lock} = 0; # just to be sure
 		$self->save_page || return;
-		$self->{repository}->close_page($self->{page});
 	}
 
-	my $rec;
-	if (ref($page) eq 'HASH') { # hist record
-		($rec, $page) = ($page, $page->{name});
-	}
-	else {
-		$rec = $self->{objects}{History}->get_record($page);
-	}
+	my ($rec, $name);
+	($name, $page, $rec) = 
+		(ref($page) eq 'HASH') ? ($page->{name}, undef, $page) :
+		ref($page)             ? ($page->name,   $page, undef) : ($page, undef, undef) ;
 	
-	if (ref $page) { # page object
-		$self->{page} = $page;
-		$page = $self->{page}->name;
-	}
-	else {
-		return warn "Warning: You tried to load an empty name.\n"
-			unless length $page;
-		
-		my $p;
-		eval { $p = $self->{repository}->open_page($page, @args) };
-		return $self->error_dialog("Could not load page $page\n\n$@")
-			if $@;
+	unless ($page) { # get object
+		return warn "Warning: You tried to load an empty name.\n" unless length $name;
 		
+		eval { $page = $self->{repository}->get_page($name) };
+		return $self->error_dialog("Could not load page $name", $@) if $@;
 		return $self->error_dialog("Page does not exist:\n$page")
-			unless defined $p
-			and not ($self->{settings}{read_only} and $p->status eq 'new');
-
-		$self->{page} = $p;
+			if ! defined $page
+			or $self->{settings}{read_only} and ! $page->exists ;
 	}
-
-	$self->{objects}{History}->set_current($self->{page});
-	eval { $self->{objects}{PageView}->load_page($self->{page}) };
+	unless ($rec) { # get history record
+		$rec = $self->{objects}{History}->get_record($name);
+	}
+	
+	$self->{objects}{History}->set_current($page);
+	eval { $self->{objects}{PageView}->load_page($page) };
 	if ($@) {
-		$self->exit_error("Could not load page $page\n\n$@");
+		$self->exit_error("Could not load page $name", $@);
 		# TODO handle more gratiously
-		# .. switching read_only would help here
 	}
 	
+	$self->{page} = $page;
 	$self->{objects}{PageView}->set_state(%$rec) if defined $rec;
-	$self->signal_emit('page_loaded', $self->{page}->name);
+	$self->signal_emit('page_loaded', $name);
 	
-	$self->{window}->set_title($self->{page}->name.' - Zim');
+	$self->{window}->set_title("$name - Zim");
 	$self->update_status();
-
-	$self->{objects}{PageView}->grab_focus;
 }
 
 =item C<save_page(FORCE)>
@@ -845,6 +927,8 @@ whether it was modified or not. Wrapper for C<Zim::Repository->save_page()>.
 # save_page() just saves it, no questions asked
 # (maybe call the first one check_save_page() ?)
 
+# TODO part of this logic belongs in the PageView component
+
 sub save_page {
 	my ($self, $force) = @_;
 	my $modified = $self->{objects}{PageView}->modified;
@@ -874,32 +958,31 @@ sub save_page {
 	unless ($@) {
 		$self->{objects}{PageView}->modified(0);
 		$self->signal_emit('page_saved', $page->name);
-		$self->update_status();
 	}
 	else { # FIXME this dialog could be a lot better
 		my $page = $self->prompt_page_dialog($page->name,
 			'Save As', 'Save Page As:', $@, 'gtk-dialog-error' );
 		return 0 unless defined $page;
 			# leave lock in position after returning
-		# FIXME check page exists
-		$self->{page} = $self->{repository}->open_page($page);
+		# FIXME check logic here !
+		$self->{page} = $self->{repository}->get_page($page);
 		$self->save_page('FORCE'); # recurs
 	}
 
+	$self->update_status();
 	$self->{save_lock} = 0;
 	return 1;
 }
 
-=item C<move_page(FROM, TO)>
+=item C<rename_page(FROM, TO)>
 
 Wrapper for C<Zim::Repository->move_page()>.
 
-Move page from FROM to TO. If TO is undefined a popup is 
-shown to ask for a page name.
+Move page from FROM to TO. If TO is undefined a dialog is shown to ask for a page name.
 
 =cut
 
-sub move_page {
+sub rename_page {
 	my ($self, $from, $to) = @_;
 
 	$from = $self->{page}->name unless defined $from;
@@ -912,15 +995,15 @@ sub move_page {
 	else { $self->save_page }
 	
 	unless (defined $to) {
-		$to = $self->prompt_move_page_dialog($from);
+		$to = $self->prompt_rename_page_dialog($from);
+		return unless defined $to;
 		$to = $self->{repository}->resolve_page($to);
+		# TODO check for things like ':'
 	}
-	
-	return unless defined $to;
 
-	# TODO check existence of target file
-	#print "Move '$from' to '$to'\n";
-	eval { $self->{repository}->move_page($from, $to) };
+	#warn "Move '$from' to '$to'\n";
+	my $page;
+	eval { $page = $self->{repository}->move_page($from, $to) };
 	return $self->error_dialog("Could not move $from to $to\n\n$@") if $@;
 
 	# TODO trigger this with signals
@@ -928,7 +1011,10 @@ sub move_page {
 	$self->{objects}{TreeView}->{_loaded} = 0;
 	$self->{objects}{TreeView}->load_index;
 
-	$self->load_page($to) if $move_current;
+	$self->load_page($page) if $move_current;
+	
+	$self->update_status;
+	return 1;
 }
 
 =item C<delete_page(PAGE)>
@@ -970,6 +1056,8 @@ sub delete_page {
 			# .. switching read_only would help here
 		}
 	}
+
+	$self->update_status;
 }
 
 =item C<message(STRING)>
@@ -1060,25 +1148,57 @@ If BOOLEAN is "-1" the pane will only be toggled on untill a page is selected.
 
 =cut
 
-sub toggle_pane {
+sub toggle_pane { # TODO does not update action state :(
 	my ($self, $pane_visible) = @_;
 	$self->{_pane_visible} = $pane_visible;
-	my $tree_view = $self->{objects}{TreeView};
+	my $widget = $self->{l_vbox};
 	my $hpaned = $self->{hpaned};
 	if ($pane_visible) {
 		$hpaned->set_position($self->{settings}{pane_pos});
-		$tree_view->widget->show_all;
-		$tree_view->grab_focus;
+		$widget->show_all;
+		$self->{notebook}->grab_focus;
 	}
 	else {
 		$self->{settings}{pane_pos} = $hpaned->get_position();
-		$tree_view->widget->hide_all;
+		$widget->hide_all;
 		$self->{objects}{PageView}->grab_focus;
 	}
 	$self->{settings}{pane_vis} = $pane_visible
 		unless $pane_visible == -1;
 }
 
+=item C<add_to_pane(WIDGET, LABEL)>
+
+Add a widget to the side pane. The label is added to the menu at the top of the side pane.
+
+=cut
+
+sub add_to_pane {
+	my ($self, $widget, $label) = @_;
+	$self->{notebook}->append_page($widget, $label);
+#	$self->{pane_combo}->append_text($label);
+	# Cross you fingers, lets hope combobox and notebook count the same way
+	my $id = $self->{pane_items}{_id}++;
+	$self->{pane_items}{lc($label)} = $id;
+}
+
+=item C<show_in_pane(NAME)>
+
+Select which widget you want to see in the side pane.
+Opens the side pane if needed.
+NAME is the same string as the label used for C<add_to_pane()>.
+
+=cut
+
+sub show_in_pane {
+	my ($self, $label) = @_;
+	my $id = $self->{pane_items}{lc($label)};
+	die "No such widget: $label" unless defined $id;
+	$self->{notebook}->set_current_page($id);
+	$self->{pane_combo}->set_active($id);
+	#$self->{actions}->get_action('TPane')->set_active(1);
+}
+
 =item C<toggle_path_bar(TYPE)>
 
 Set or cycle the type of the pathbar.
@@ -1141,6 +1261,7 @@ sub error_dialog {
 	print STDERR "zim: $text2\n";
 	$dialog->run;
 	$dialog->destroy;
+	return undef;
 }
 
 =item C<repository_lock_dialog>
@@ -1233,6 +1354,11 @@ sub prompt_page_dialog {
 	$entry->set_text($page);
 	$hbox->add($entry);
 	
+	#my $completion = Gtk2::EntryCompletion->new;
+	#$completion->set_model($self->TreeView->get_model());
+	#$completion->set_text_column(1);
+	#$entry->set_completion($completion);
+	
 	$dialog->show_all;
 	if ($dialog->run eq 'ok') { $page = $entry->get_text }
 	else                      { $page = undef            }
@@ -1242,14 +1368,14 @@ sub prompt_page_dialog {
 	return $page;
 }
 
-=item C<prompt_move_page_dialog>
+=item C<prompt_rename_page_dialog>
 
 =cut
 
-sub prompt_move_page_dialog { # TODO base this dialog on prompt_page_dialog()
+sub prompt_rename_page_dialog { # TODO base this dialog on prompt_page_dialog()
 	my ($self, $page) = @_;
 	
-	my $title = "Move page";
+	my $title = "Rename page";
 	my $dialog = Gtk2::Dialog->new(
 		$title, $self->{window},
 	       	[qw/modal destroy-with-parent no-separator/],
@@ -1260,7 +1386,7 @@ sub prompt_move_page_dialog { # TODO base this dialog on prompt_page_dialog()
 	$dialog->set_icon($self->{window}->get_icon);
 	$dialog->set_default_response('ok');
 
-	my $move_button = $self->new_button('gtk-save', '_Move');
+	my $move_button = $self->new_button('gtk-save', '_Rename');
 	$dialog->add_action_widget($move_button, 'ok');
 	# $dialog->set_default_response('ok'); FIXME
 
@@ -1401,7 +1527,7 @@ sub about_dialog { # TODO use Gtk2::AboutDialog for gtk > 2.6
 		Gtk2::Image->new_from_file($self->{settings}{icon_file}) );
 
 	my $text = $LONG_VERSION;
-	$text =~ s/^(.*)$/<b>$1<\/b>/m;
+	$text =~ s/^(.*)$/\n<b>$1<\/b>/m;
 	my $label = Gtk2::Label->new();
 	$label->set_markup($text);
 	$label->set_justify('center');
@@ -1416,27 +1542,37 @@ sub about_dialog { # TODO use Gtk2::AboutDialog for gtk > 2.6
 
 =item C<show_help(PAGE)>
 
-Spawns a new zim process showing the documentation. PAGE is optional.
+Show a window showing the documentation. PAGE is optional.
 
 =cut
 
 sub show_help {
 	my ($self, $page) = @_;
+	my @args = ('--doc', $page ? $page : ());
+	$self->exec_new_window(@args);
+}
+
+=item  C<exec_new_window(..)>
+
+Executes a new process for $0, this gives a detached window.
+Any arguments are passed on to the new process.
+
+=cut
+
+sub exec_new_window {
+	my ($self, @args) = @_;
 	
-	unless (fork) { # child process
-		no warnings;
-		@{$self->{settings}}{'width', 'height'} = $self->{window}->get_size;
-		$self->save_config(); # maybe we are read_only too
-		my @exec = ($0, '--doc', $page ? $page : ());
-		exec @exec;
-		# backup procedure to find perl
-		# when $0 is not executable for some reason
-		eval 'use Config';
-		exit 1 if $@;
-		my $perl = $Config{perl5} || 'perl';
-		exec @exec;
-		exit 1; # just to be sure
-	}
+	@{$self->{settings}}{'width', 'height'} = $self->{window}->get_size;
+	$self->save_config(); # maybe we are read_only too
+	fork && return;
+	#warn "Trying to execute: $0 @args\n";
+
+	# child process
+	no warnings;
+	exec $0, @args;
+	# backup procedure when $0 did not work
+	exec $^X, $0, @args;
+	exit 1; # just to be sure
 }
 
 =item C<filechooser_dialog(FILE, TITLE)>
@@ -1472,62 +1608,155 @@ sub filechooser_dialog {
 	return $file;
 }
 
-=item C<prompt_root_dialog>
+=item C<prompt_repository_dialog>
 
 =cut
 
-sub prompt_root_dialog {
-	my ($self, $page, $title, $prompt, $text, $icon) = @_;
+sub prompt_repository_dialog { # TODO improve visual layout
+	my $self = shift;
 
 	my $dialog = Gtk2::Dialog->new(
-		"Open directory - Zim", undef,
-	       	[qw/modal destroy-with-parent no-separator/],
+		"Open repository - Zim", $self->{window},
+	       	[qw/destroy-with-parent no-separator/],
 		'gtk-cancel' => 'cancel',
 		'gtk-open'   => 'ok',
 	);
+	$dialog->set_default_size(100,250);
+	$dialog->set_border_width(5);
+	$dialog->set_icon(
+		Gtk2::Gdk::Pixbuf->new_from_file($self->{settings}{icon_file}) );
+	$dialog->set_default_response('ok');
+	
+	$dialog->vbox->set_spacing(5);
+	$dialog->vbox->pack_start(
+		Gtk2::Label->new('Please choose a repository'), 0,1,0 );
+	
+	my $list = Gtk2::SimpleList->new('Repository' => 'text');
+	$list->set_headers_visible(0);
+	$list->get_selection->set_mode('browse');
+	$list->signal_connect(row_activated => sub { $dialog->response('ok') });
+	$dialog->vbox->pack_start($list, 1,1,0);
+	
+	# Read list - FIXME also look in other xdg_config dirs
+	my %source;
+	my $file = File::Spec->catfile(
+		File::BaseDir->xdg_config_home, 'zim', 'repositories.list' );
+	if (-f $file) {
+		open IN, $file or die "Could not read file: $file\n";
+		while (<IN>) {
+			chomp;
+			my ($name, $source) = split '=', $_, 2;
+			next unless $name =~ /\S/;
+			$source{$name} = $source;
+			push @{$list->{data}}, $name;
+		}
+		close IN;
+	}
+	
+	my $hbox = Gtk2::HBox->new(1,10);
+	my $add_button = Gtk2::Button->new_from_stock('gtk-add');
+	my $rem_button = Gtk2::Button->new_from_stock('gtk-remove');
+	$hbox->add($add_button);
+	$hbox->add($rem_button);
+	$dialog->vbox->pack_start($hbox, 0,1,0);
+	
+	my $modified = 0;
+	$add_button->signal_connect(clicked => sub {
+			my ($name, $source) = $self->prompt_new_repository_dialog($dialog);
+			return unless defined $name;
+			$source{$name} = $source;
+			push @{$list->{data}}, $name;
+			$modified = 1;
+		} );
+	$rem_button->signal_connect(clicked => sub {
+			my ($i) = $list->get_selected_indices;
+			splice @{$list->{data}}, $i, 1;
+			$modified = 1;
+		} );
+	
+	$dialog->show_all;
+	my $repository;
+	RUN_REP_DIALOG:
+	if ($dialog->run eq 'ok') {
+		my ($i) = $list->get_selected_indices;
+		unless(defined $i) {
+			$self->error_dialog("Please select a repository first.\n");
+			goto RUN_REP_DIALOG;
+		}
+		
+		my ($name) = @{$list->{data}[$i]};
+		$repository = $source{$name};
+		#warn "You selected \"$name\" ($i) => $repository\n";
+	}
+	else { $modified = 0 } # Don't save at cancel
+	$dialog->destroy;
+
+	# Save list
+	if ($modified) {
+		open OUT, ">$file" or die "Could not write file: $file\n";
+		for (@{$list->{data}}) {
+			my ($name) = @$_;
+			print OUT "$name=$source{$name}\n";
+		}
+		close OUT or die "Could not write file: $file\n";
+	}
+	
+	return $repository;
+}
+
+=item C<prompt_new_repository_dialog(WINDOW)>
+
+=cut
+
+sub prompt_new_repository_dialog { # TODO add "browse" button for dir
+	my $self = shift;
+	my $window = shift || $self->{window};
+	
+	my $dialog = Gtk2::Dialog->new(
+		'New repository - Zim', $window,
+	       	[qw/modal destroy-with-parent no-separator/],
+		'gtk-cancel' => 'cancel',
+		'gtk-ok'     => 'ok',
+	);
 	$dialog->set_resizable(0);
 	$dialog->set_border_width(5);
 	$dialog->set_icon(
 		Gtk2::Gdk::Pixbuf->new_from_file($self->{settings}{icon_file}) );
 	$dialog->set_default_response('ok');
-	
-	$dialog->vbox->add(
-		Gtk2::Image->new_from_file($self->{settings}{icon_file}) );
 
-	$dialog->vbox->add( Gtk2::Label->new( << 'EOT' ) );
-Welcom to Zim.
+	$dialog->vbox->set_spacing(5);
+	$dialog->vbox->add( Gtk2::Label->new(
+		"Please give a new directory\nto store your pages" ) );
+	
+	my $table = Gtk2::Table->new(2, 2);
+	$table->set_border_width(5);
+	$table->set_row_spacings(5);
+	$table->set_col_spacings(5);
+	$dialog->vbox->add($table);
+	
+	my $dir_label = Gtk2::Label->new('Directory:');
+	my $name_label = Gtk2::Label->new('Name:');
+	my $dir_entry = Gtk2::Entry->new();
+	$dir_entry->signal_connect(activate => sub { $dialog->response('ok') });
+	my $name_entry = Gtk2::Entry->new();
+	$name_entry->signal_connect(activate => sub { $dialog->response('ok') });
+	$table->attach_defaults($dir_label, 0,1, 0,1);
+	$table->attach_defaults($dir_entry, 1,2, 0,1);
+	$table->attach_defaults($name_label, 0,1, 1,2);
+	$table->attach_defaults($name_entry, 1,2, 1,2);
 
-Zim needs a directory to store its wiki page.
-Please enter a default directory below.
-
-You can open other directories by giving them as commandline
-a argument to zim(1).
-
-If you want to change the default directory later you need to
-edit the "default_root" option in the config file.
-(TODO: add a nice dialog for changing config options)
-EOT
-	
-	my $hbox  = Gtk2::HBox->new(0, 5);
-	$hbox->set_border_width(5);
-	$dialog->vbox->add($hbox);
-	
-	my $label = Gtk2::Label->new("Open directory: ");
-	$hbox->add($label);
-	
-	my $entry = Gtk2::Entry->new();
-	$entry->signal_connect(activate => sub { $dialog->response('ok') });
-	$entry->set_text("$ENV{HOME}/zim_pages/");
-	$hbox->add($entry);
-	
-	my $root;
 	$dialog->show_all;
-	if ($dialog->run eq 'ok') { $root = $entry->get_text }
-	else                      { $root = undef            }
+	my ($name, $dir);
+	if ($dialog->run eq 'ok') {
+		$name = $name_entry->get_text;
+		$dir = $dir_entry->get_text;
+	}
 	$dialog->destroy;
 
-	$root = undef unless $root =~ /\S/;
-	return $root;
+	if ($dir !~ /\S/) { $name = undef }
+	elsif ($name !~ /\S/) { $name = $dir }
+	
+	return ($name, $dir);
 }
 
 =item C<prompt_browser_dialog>
@@ -1541,8 +1770,8 @@ sub prompt_browser_dialog {
 	my $dialog = Gtk2::Dialog->new(
 		$title, $self->{window},
 	       	[qw/modal destroy-with-parent no-separator/],
-		'gtk-ok'     => 'ok',
 		'gtk-cancel' => 'cancel',
+		'gtk-ok'     => 'ok',
 	);
 	$dialog->set_resizable(0);
 	$dialog->set_border_width(5);
