@@ -6,9 +6,10 @@ use Gtk2;
 use Gtk2::Pango;               # pango constants
 use Gtk2::Ex::HyperTextView;   # custom widget
 use Gtk2::Ex::HyperTextBuffer; #    "     "
+#use Gtk2::Ex::OpenWith;        # for file:// links
 use POSIX qw(strftime);
 
-our $VERSION = '0.10';
+our $VERSION = '0.12';
 
 # TODO move more logic into Gtk2::Ex::HyperTextBuffer !
 
@@ -35,27 +36,30 @@ our $LINK_ICON = Gtk2->CHECK_VERSION(2, 6, 0) ? 'gtk-connect' : 'gtk-convert';
 	# gtk-connect stock item was introduced in 2.6.0
 
 my @ui_actions = (
-	# name,          stock id,         label,            accelerator,       tooltip
-	[ 'Head1',       undef,            'Head _1',        '<ctrl>1',         'Heading 1',     \&on_tag         ],
-	[ 'Head2',       undef,            'Head _2',        '<ctrl>2',         'Heading 2',     \&on_tag         ],
-	[ 'Head3',       undef,            'Head _3',        '<ctrl>3',         'Heading 3',     \&on_tag         ],
-	[ 'Head4',       undef,            'Head _4',        '<ctrl>4',         'Heading 4',     \&on_tag         ],
-	[ 'Head5',       undef,            'Head _5',        '<ctrl>5',         'Heading 5',     \&on_tag         ],
-	[ 'Normal',      undef,            '_Normal',        '<ctrl>6',         'Normal',        \&on_tag         ],
-	[ 'Bold',        'gtk-bold',       '_Bold',          '<ctrl>B',         'Bold',          \&on_tag         ],
-	[ 'Italic',      'gtk-italic',     '_Italic',        '<ctrl>I',         'Italic',        \&on_tag         ],
-	[ 'Underline',   'gtk-underline',  '_Underline',     '<ctrl>U',         'Underline',     \&on_tag         ],
-	[ 'Verbatim',    undef,            '_Verbatim',      '<ctrl>T',         'Verbatim',      \&on_tag         ],
-	[ 'Undo',        'gtk-undo',       '_Undo',          '<ctrl>Z',         'Undo',          \&on_Undo        ],
-	[ 'Redo',        'gtk-redo',       '_Redo',          '<ctrl><shift>Z',  'Redo',          \&on_Redo        ],
-	[ 'Cut',         'gtk-cut',        'Cu_t',           '<ctrl>X',         'Cut',           \&on_clipboard   ],
-	[ 'Copy',        'gtk-copy',       '_Copy',          '<ctrl>C',         'Copy',          \&on_clipboard   ],
-	[ 'Paste',       'gtk-paste',      '_Paste',         '<ctrl>V',         'Paste',         \&on_clipboard   ],
-	[ 'Delete',      'gtk-delete',     'Delete',         'Delete',          'Delete',        \&on_Delete      ],
-	[ 'EditLink',    'gtk-properties', '_Link',          '<ctrl>E',         'Edit link',     \&on_EditLink    ],
-	[ 'InsertDate',  undef,            '_Date and Time', '<ctrl>D',         'Insert date',   \&on_InsertDate  ],
-	[ 'InsertLink',  $LINK_ICON,       '_Link...',       undef,             'Insert link',   \&on_EditLink    ],
-	[ 'InsertImage', undef,            '_Image...',      undef,             'Insert image',  \&on_InsertImage ],
+	# name,          stock id,           label,            accelerator,       tooltip
+	[ 'Head1',       undef,              'Head _1',        '<ctrl>1',         'Heading 1',     \&on_tag         ],
+	[ 'Head2',       undef,              'Head _2',        '<ctrl>2',         'Heading 2',     \&on_tag         ],
+	[ 'Head3',       undef,              'Head _3',        '<ctrl>3',         'Heading 3',     \&on_tag         ],
+	[ 'Head4',       undef,              'Head _4',        '<ctrl>4',         'Heading 4',     \&on_tag         ],
+	[ 'Head5',       undef,              'Head _5',        '<ctrl>5',         'Heading 5',     \&on_tag         ],
+	[ 'Normal',      undef,              '_Normal',        '<ctrl>6',         'Normal',        \&on_tag         ],
+	[ 'Bold',        'gtk-bold',         '_Bold',          '<ctrl>B',         'Bold',          \&on_tag         ],
+	[ 'Italic',      'gtk-italic',       '_Italic',        '<ctrl>I',         'Italic',        \&on_tag         ],
+	[ 'Underline',   'gtk-underline',    '_Underline',     '<ctrl>U',         'Underline',     \&on_tag         ],
+	[ 'Strike',      'gtk-strikethrough','Stri_ke',        '<ctrl>K',         'Strike',        \&on_tag         ],
+	[ 'Verbatim',    undef,              '_Verbatim',      '<ctrl>T',         'Verbatim',      \&on_tag         ],
+	[ 'SplitPage',   undef,              'Spli_t page',    undef,             'Split page',    \&on_SplitPage   ],
+	[ 'MergePage',   undef,              '_Merge page',    undef,             'Merge page',    \&on_MergePage   ],
+	[ 'Undo',        'gtk-undo',         '_Undo',          '<ctrl>Z',         'Undo',          \&on_Undo        ],
+	[ 'Redo',        'gtk-redo',         '_Redo',          '<ctrl><shift>Z',  'Redo',          \&on_Redo        ],
+	[ 'Cut',         'gtk-cut',          'Cu_t',           '<ctrl>X',         'Cut',           \&on_clipboard   ],
+	[ 'Copy',        'gtk-copy',         '_Copy',          '<ctrl>C',         'Copy',          \&on_clipboard   ],
+	[ 'Paste',       'gtk-paste',        '_Paste',         '<ctrl>V',         'Paste',         \&on_clipboard   ],
+	[ 'Delete',      'gtk-delete',       'Delete',         'Delete',          'Delete',        \&on_Delete      ],
+	[ 'EditLink',    'gtk-properties',   '_Link',          '<ctrl>E',         'Edit link',     \&on_EditLink    ],
+	[ 'InsertDate',  undef,              '_Date and Time', '<ctrl>D',         'Insert date',   \&on_InsertDate  ],
+	[ 'InsertLink',  $LINK_ICON,         '_Link...',       undef,             'Insert link',   \&on_EditLink    ],
+	[ 'InsertImage', undef,              '_Image...',      undef,             'Insert image',  \&on_InsertImage ],
 );
 
 my @ui_actions_ro = (
@@ -65,6 +69,13 @@ my @ui_actions_ro = (
 	[ 'FindPrev',    undef,            'Find Pre_vious', '<ctrl><shift>G',  'Find previous', \&on_FindPrev    ],
 );
 
+#		<menu action='FileMenu'>
+#			<placeholder name='FileMods'>
+#				<menuitem action='SplitPage'/>
+#				<menuitem action='MergePage'/>
+#			</placeholder>
+#		</menu>
+
 my $ui_layout = q{<ui>
 	<menubar name='MenuBar'>
 		<menu action='EditMenu'>
@@ -96,6 +107,7 @@ my $ui_layout = q{<ui>
 			<menuitem action='Bold'/>
 			<menuitem action='Italic'/>
 			<menuitem action='Underline'/>
+			<menuitem action='Strike'/>
 			<menuitem action='Verbatim'/>
 			<separator/>
 			<menuitem action='Link'/>
@@ -125,6 +137,273 @@ my $ui_layout_ro = q{<ui>
 	<accelerator action='Link'/>
 </ui>};
 
+## Table below copied from HTML::Entities by Gisle Aas ##
+our %entity2char = (
+ # Some normal chars that have special meaning in SGML context
+ amp    => '&',  # ampersand 
+'gt'    => '>',  # greater than
+'lt'    => '<',  # less than
+ quot   => '"',  # double quote
+ apos   => "'",  # single quote
+
+ # PUBLIC ISO 8879-1986//ENTITIES Added Latin 1//EN//HTML
+ AElig	=> chr(198),  # capital AE diphthong (ligature)
+ Aacute	=> chr(193),  # capital A, acute accent
+ Acirc	=> chr(194),  # capital A, circumflex accent
+ Agrave	=> chr(192),  # capital A, grave accent
+ Aring	=> chr(197),  # capital A, ring
+ Atilde	=> chr(195),  # capital A, tilde
+ Auml	=> chr(196),  # capital A, dieresis or umlaut mark
+ Ccedil	=> chr(199),  # capital C, cedilla
+ ETH	=> chr(208),  # capital Eth, Icelandic
+ Eacute	=> chr(201),  # capital E, acute accent
+ Ecirc	=> chr(202),  # capital E, circumflex accent
+ Egrave	=> chr(200),  # capital E, grave accent
+ Euml	=> chr(203),  # capital E, dieresis or umlaut mark
+ Iacute	=> chr(205),  # capital I, acute accent
+ Icirc	=> chr(206),  # capital I, circumflex accent
+ Igrave	=> chr(204),  # capital I, grave accent
+ Iuml	=> chr(207),  # capital I, dieresis or umlaut mark
+ Ntilde	=> chr(209),  # capital N, tilde
+ Oacute	=> chr(211),  # capital O, acute accent
+ Ocirc	=> chr(212),  # capital O, circumflex accent
+ Ograve	=> chr(210),  # capital O, grave accent
+ Oslash	=> chr(216),  # capital O, slash
+ Otilde	=> chr(213),  # capital O, tilde
+ Ouml	=> chr(214),  # capital O, dieresis or umlaut mark
+ THORN	=> chr(222),  # capital THORN, Icelandic
+ Uacute	=> chr(218),  # capital U, acute accent
+ Ucirc	=> chr(219),  # capital U, circumflex accent
+ Ugrave	=> chr(217),  # capital U, grave accent
+ Uuml	=> chr(220),  # capital U, dieresis or umlaut mark
+ Yacute	=> chr(221),  # capital Y, acute accent
+ aacute	=> chr(225),  # small a, acute accent
+ acirc	=> chr(226),  # small a, circumflex accent
+ aelig	=> chr(230),  # small ae diphthong (ligature)
+ agrave	=> chr(224),  # small a, grave accent
+ aring	=> chr(229),  # small a, ring
+ atilde	=> chr(227),  # small a, tilde
+ auml	=> chr(228),  # small a, dieresis or umlaut mark
+ ccedil	=> chr(231),  # small c, cedilla
+ eacute	=> chr(233),  # small e, acute accent
+ ecirc	=> chr(234),  # small e, circumflex accent
+ egrave	=> chr(232),  # small e, grave accent
+ eth	=> chr(240),  # small eth, Icelandic
+ euml	=> chr(235),  # small e, dieresis or umlaut mark
+ iacute	=> chr(237),  # small i, acute accent
+ icirc	=> chr(238),  # small i, circumflex accent
+ igrave	=> chr(236),  # small i, grave accent
+ iuml	=> chr(239),  # small i, dieresis or umlaut mark
+ ntilde	=> chr(241),  # small n, tilde
+ oacute	=> chr(243),  # small o, acute accent
+ ocirc	=> chr(244),  # small o, circumflex accent
+ ograve	=> chr(242),  # small o, grave accent
+ oslash	=> chr(248),  # small o, slash
+ otilde	=> chr(245),  # small o, tilde
+ ouml	=> chr(246),  # small o, dieresis or umlaut mark
+ szlig	=> chr(223),  # small sharp s, German (sz ligature)
+ thorn	=> chr(254),  # small thorn, Icelandic
+ uacute	=> chr(250),  # small u, acute accent
+ ucirc	=> chr(251),  # small u, circumflex accent
+ ugrave	=> chr(249),  # small u, grave accent
+ uuml	=> chr(252),  # small u, dieresis or umlaut mark
+ yacute	=> chr(253),  # small y, acute accent
+ yuml	=> chr(255),  # small y, dieresis or umlaut mark
+
+ # Some extra Latin 1 chars that are listed in the HTML3.2 draft (21-May-96)
+ copy   => chr(169),  # copyright sign
+ reg    => chr(174),  # registered sign
+ nbsp   => chr(160),  # non breaking space
+
+ # Additional ISO-8859/1 entities listed in rfc1866 (section 14)
+ iexcl  => chr(161),
+ cent   => chr(162),
+ pound  => chr(163),
+ curren => chr(164),
+ yen    => chr(165),
+ brvbar => chr(166),
+ sect   => chr(167),
+ uml    => chr(168),
+ ordf   => chr(170),
+ laquo  => chr(171),
+'not'   => chr(172),    # not is a keyword in perl
+ shy    => chr(173),
+ macr   => chr(175),
+ deg    => chr(176),
+ plusmn => chr(177),
+ sup1   => chr(185),
+ sup2   => chr(178),
+ sup3   => chr(179),
+ acute  => chr(180),
+ micro  => chr(181),
+ para   => chr(182),
+ middot => chr(183),
+ cedil  => chr(184),
+ ordm   => chr(186),
+ raquo  => chr(187),
+ frac14 => chr(188),
+ frac12 => chr(189),
+ frac34 => chr(190),
+ iquest => chr(191),
+'times' => chr(215),    # times is a keyword in perl
+ divide => chr(247),
+
+ ( $] > 5.007 ? (
+   OElig    => chr(338),
+   oelig    => chr(339),
+   Scaron   => chr(352),
+   scaron   => chr(353),
+   Yuml     => chr(376),
+   fnof     => chr(402),
+   circ     => chr(710),
+   tilde    => chr(732),
+   Alpha    => chr(913),
+   Beta     => chr(914),
+   Gamma    => chr(915),
+   Delta    => chr(916),
+   Epsilon  => chr(917),
+   Zeta     => chr(918),
+   Eta      => chr(919),
+   Theta    => chr(920),
+   Iota     => chr(921),
+   Kappa    => chr(922),
+   Lambda   => chr(923),
+   Mu       => chr(924),
+   Nu       => chr(925),
+   Xi       => chr(926),
+   Omicron  => chr(927),
+   Pi       => chr(928),
+   Rho      => chr(929),
+   Sigma    => chr(931),
+   Tau      => chr(932),
+   Upsilon  => chr(933),
+   Phi      => chr(934),
+   Chi      => chr(935),
+   Psi      => chr(936),
+   Omega    => chr(937),
+   alpha    => chr(945),
+   beta     => chr(946),
+   gamma    => chr(947),
+   delta    => chr(948),
+   epsilon  => chr(949),
+   zeta     => chr(950),
+   eta      => chr(951),
+   theta    => chr(952),
+   iota     => chr(953),
+   kappa    => chr(954),
+   lambda   => chr(955),
+   mu       => chr(956),
+   nu       => chr(957),
+   xi       => chr(958),
+   omicron  => chr(959),
+   pi       => chr(960),
+   rho      => chr(961),
+   sigmaf   => chr(962),
+   sigma    => chr(963),
+   tau      => chr(964),
+   upsilon  => chr(965),
+   phi      => chr(966),
+   chi      => chr(967),
+   psi      => chr(968),
+   omega    => chr(969),
+   thetasym => chr(977),
+   upsih    => chr(978),
+   piv      => chr(982),
+   ensp     => chr(8194),
+   emsp     => chr(8195),
+   thinsp   => chr(8201),
+   zwnj     => chr(8204),
+   zwj      => chr(8205),
+   lrm      => chr(8206),
+   rlm      => chr(8207),
+   ndash    => chr(8211),
+   mdash    => chr(8212),
+   lsquo    => chr(8216),
+   rsquo    => chr(8217),
+   sbquo    => chr(8218),
+   ldquo    => chr(8220),
+   rdquo    => chr(8221),
+   bdquo    => chr(8222),
+   dagger   => chr(8224),
+   Dagger   => chr(8225),
+   bull     => chr(8226),
+   hellip   => chr(8230),
+   permil   => chr(8240),
+   prime    => chr(8242),
+   Prime    => chr(8243),
+   lsaquo   => chr(8249),
+   rsaquo   => chr(8250),
+   oline    => chr(8254),
+   frasl    => chr(8260),
+   euro     => chr(8364),
+   image    => chr(8465),
+   weierp   => chr(8472),
+   real     => chr(8476),
+   trade    => chr(8482),
+   alefsym  => chr(8501),
+   larr     => chr(8592),
+   uarr     => chr(8593),
+   rarr     => chr(8594),
+   darr     => chr(8595),
+   harr     => chr(8596),
+   crarr    => chr(8629),
+   lArr     => chr(8656),
+   uArr     => chr(8657),
+   rArr     => chr(8658),
+   dArr     => chr(8659),
+   hArr     => chr(8660),
+   forall   => chr(8704),
+   part     => chr(8706),
+   exist    => chr(8707),
+   empty    => chr(8709),
+   nabla    => chr(8711),
+   isin     => chr(8712),
+   notin    => chr(8713),
+   ni       => chr(8715),
+   prod     => chr(8719),
+   sum      => chr(8721),
+   minus    => chr(8722),
+   lowast   => chr(8727),
+   radic    => chr(8730),
+   prop     => chr(8733),
+   infin    => chr(8734),
+   ang      => chr(8736),
+  'and'     => chr(8743),
+  'or'      => chr(8744),
+   cap      => chr(8745),
+   cup      => chr(8746),
+  'int'     => chr(8747),
+   there4   => chr(8756),
+   sim      => chr(8764),
+   cong     => chr(8773),
+   asymp    => chr(8776),
+  'ne'      => chr(8800),
+   equiv    => chr(8801),
+  'le'      => chr(8804),
+  'ge'      => chr(8805),
+  'sub'     => chr(8834),
+   sup      => chr(8835),
+   nsub     => chr(8836),
+   sube     => chr(8838),
+   supe     => chr(8839),
+   oplus    => chr(8853),
+   otimes   => chr(8855),
+   perp     => chr(8869),
+   sdot     => chr(8901),
+   lceil    => chr(8968),
+   rceil    => chr(8969),
+   lfloor   => chr(8970),
+   rfloor   => chr(8971),
+   lang     => chr(9001),
+   rang     => chr(9002),
+   loz      => chr(9674),
+   spades   => chr(9824),
+   clubs    => chr(9827),
+   hearts   => chr(9829),
+   diams    => chr(9830),
+ ) : ())
+);
+
 =item C<new(app => PARENT)>
 
 Simple constructor.
@@ -184,7 +463,7 @@ sub init { # called by new()
 	$htext->signal_connect(link_leave =>
 		sub { $self->{app}->pop_status('link')                    }  );
 	$htext->signal_connect(toggle_overwrite => \&on_toggle_overwrite, $self);
-	$htext->signal_connect(populate_popup => \&on_populate_popup);
+	$htext->signal_connect(populate_popup => \&on_populate_popup, $self);
 	$htext->signal_connect(key_press_event => \&on_key_press_event, $self);
 	my $reset_em = sub {
 		$self->{buffer}->reset_edit_mode if $self->{buffer};
@@ -201,7 +480,15 @@ sub init { # called by new()
 	$hbox->signal_connect(key_press_event => \&on_key_press_event_hbox);
 	$vbox->pack_start($hbox, 0, 1, 3);
 	$self->{hbox} = $hbox;
-
+	
+	my $close_button = Gtk2::Button->new;
+	$close_button->set_relief('none');
+	$close_button->add(Gtk2::Alignment->new(0.5, 0.5, 0, 0));
+	$close_button->child->add(
+		Gtk2::Image->new_from_stock('gtk-close', 'small-toolbar') );
+	$close_button->signal_connect(clicked => sub { $hbox->hide } );
+	$hbox->pack_end($close_button, 0,1,0);
+	
 	$hbox->pack_start( Gtk2::Label->new(' Find: '), 0,1,0);
 	
 	my $entry = Gtk2::Entry->new();
@@ -254,6 +541,25 @@ sub on_tag {
 
 sub on_Link { pop->apply_link }
 
+sub on_SplitPage {
+	my $self = pop;
+	# TODO prompt level
+	$self->{app}->save_page;
+	$self->{app}{repository}->split_page($self->{app}{page}, 1);
+	$self->{app}->TreeView->{_loaded} = 0;
+	$self->{app}->TreeView->load_index;
+	$self->{app}->reload;
+}
+
+sub on_MergePage {
+	my $self = pop;
+	$self->{app}->save_page;
+	$self->{app}{repository}->merge_page($self->{app}{page});
+	$self->{app}->TreeView->{_loaded} = 0;
+	$self->{app}->TreeView->load_index;
+	$self->{app}->reload;
+}
+
 sub on_Undo { pop->undo }
 
 sub on_Redo { pop->redo }
@@ -349,24 +655,10 @@ sub find {
 	}
 	
 	#print "found $string at offset ".$iter->get_offset."\n";
-	$self->set_selection($start, $end);
+	$self->{buffer}->select_range($start, $end);
 	$self->{htext}->scroll_mark_onscreen( $buffer->get_insert );
 }
 
-=item C<set_selection(START, END)>
-
-Select a region of the buffer.
-
-=cut
-
-sub set_selection {
-	my ($self, $start, $end) = @_;
-	my $buffer = $self->{htext}->get_buffer;
-	$buffer->place_cursor($start);
-	$buffer->move_mark( $buffer->get_selection_bound, $end );
-#	$buffer->select_range($start, $end); # Since 2.4 FIXME
-}
-
 =item C<show_find()>
 
 Show the search bar.
@@ -399,39 +691,70 @@ sub on_toggle_overwrite {
 	$self->{app}->update_status();
 }
 
-sub on_populate_popup { # FIXME add this code to HyperTextView
+sub on_populate_popup {
        	# add items to context menu
-	my ($htext, $menu) = @_;
+	my ($htext, $menu, $self) = @_;
 	my $iter = $htext->get_iter_at_pointer;
 	return unless $iter;
 	
-	my $string;
+	my ($copy_string, $edit_string, $code);
 	my $link = $htext->get_link_at_iter($iter);
+	my ($type, $source);
 	if (defined $link) {
+		$type = 'link';
 		$link = $$link[1] if ref($link) eq 'ARRAY';
-		$string = ($link =~ s/^mailto:\/*//)
-			? 'Copy _Email Address'
+		$source = $link;
+		$copy_string = ($source =~ s/^mailto:\/*//)
+			? 'Copy Email Address'
 			: 'Copy _Link' ;
+		$edit_string = '_Edit link';
+		$code = sub {$self->edit_link_dialog($iter)};
 	}
-	else {
-		my $pixbuf = $iter->get_pixbuf;
-		return unless $pixbuf and defined $pixbuf->{image_src};
-		$link = $pixbuf->{image_src};
-		$string = 'Copy Image _Location';
+	elsif (my $pixbuf = $iter->get_pixbuf) {
+		return unless defined $pixbuf->{image_src};
+		$type = 'image';
+		$source = $pixbuf->{image_src};
+		$copy_string = 'Copy Image _Location';
+		$edit_string = '_Edit properties';
+		$code = sub {$self->edit_image_dialog($iter)};
 	}
+	else { return }
 	
 	my $seperator = Gtk2::MenuItem->new();
 	$seperator->show;
 	$menu->prepend($seperator);
 	
-	my $item = Gtk2::MenuItem->new($string);
-	$item->signal_connect(activate => sub {
+	if ($type eq 'link') { # TODO image support
+		my $edit_item = Gtk2::MenuItem->new($edit_string);
+		$edit_item->show;
+		$edit_item->signal_connect_swapped(activate => $code, $self);
+		$menu->prepend($edit_item);
+	}
+
+	my $copy_item = Gtk2::MenuItem->new($copy_string);
+	$copy_item->signal_connect(activate => sub {
 		my $clipboard = Gtk2::Clipboard->get(
 			Gtk2::Gdk::Atom->new('PRIMARY') );
-		$clipboard->set_text($link);
+		$clipboard->set_text($source);
 	} );
-	$item->show;
-	$menu->prepend($item);
+	$copy_item->show;
+	$menu->prepend($copy_item);
+
+	if ($type eq 'image') { # or $link =~ m#^file://#) {
+#		Gtk2::Ex::OpenWith->populate_menu($menu, $link);
+#		FIXME check versions etc.
+	}
+	else {
+		$seperator = Gtk2::MenuItem->new();
+		$seperator->show;
+		$menu->prepend($seperator);
+	
+		my $open_item = Gtk2::MenuItem->new('_Open');
+		$open_item->signal_connect(activate =>
+			sub { $self->{app}->link_clicked($link) } );
+		$open_item->show;
+		$menu->prepend($open_item);
+	}
 }
 
 sub on_key_press_event { # some extra keybindings
@@ -449,6 +772,8 @@ sub on_key_press_event { # some extra keybindings
 		my $buffer = $htext->get_buffer;
 		my $iter = $buffer->get_iter_at_mark($buffer->get_insert());
 		return 1 if defined $htext->click_if_link_at_iter($iter);
+		$self->parse_word($iter); # end-of-line is also end-of-word
+		$iter = $buffer->get_iter_at_mark($buffer->get_insert());
 		$self->parse_line($iter) or return 0;
 		$htext->scroll_mark_onscreen( $buffer->get_insert );
 		return 1;
@@ -607,16 +932,17 @@ sub load_page {
 	}
 	$self->{app}->Spell->attach($self->{htext}) if $use_spell;
 	$self->{undo_lock} = 0;
-	$buffer->set_modified(0);
 
 	$buffer->place_cursor(
-		$buffer->get_iter_at_offset(0) )
-			unless $page->status eq 'new';
+		$buffer->get_iter_at_offset(0) ) if $page->exists;
+		# new page can have template, place cursor below heading etc.
 	
 	$self->{undo} = [];
 	$self->{redo} = [];
 	
 	$self->{htext}->scroll_mark_onscreen( $buffer->get_insert );
+
+	$buffer->set_modified(0);
 }
 
 =item C<modified()>
@@ -681,18 +1007,24 @@ sub parse_line {
 	my $lf = $buffer->get_iter_at_line( $iter->get_line );
 	my $line = $buffer->get_text($lf, $iter, 0);
 	#print ">>$line<<\n";
-	if ($line =~ s/^(=+)\s+(\S)/$2/) { # heading
+	if ($line =~ s/^(=+)\s*(\w)/$2/) { # heading
 		my $h = length($1); # no (7 - x) monkey bussiness here
 		$h = 5 if $h > 5;
 		$line =~ s/\s+=+\s*$//;
 		$buffer->delete($lf, $iter);
 		$buffer->insert_with_tags_by_name($lf, $line, "head$h");
 	}
-	elsif ($line =~ /^\s*[\*\x{2022}]\s+$/) { # empty bullet
+	elsif ($line =~ /^\s*([\*\x{2022}]|(\d+|\w)\W*)\s+$/) {
+		# empty bullet or list item
 		$buffer->delete($lf, $iter);
 	}
-	elsif ($line =~ /^(\s*\W+\s+|\s+)/) { # auto indenting
-		$buffer->insert($iter, "\n$1");
+	elsif ($line =~ /^(\s*(\d*|\w?)\W*\s+)/) { # auto indenting + lists
+		my ($indent, $number) = ($1, $2);
+		if (length $number) { # numbered list
+			$number = ($number =~ /\d/) ? $number+1 : chr(ord($number)+1);
+			$indent =~ s/^(\s*)(\d+|\w)/$1$number/;
+		}
+		$buffer->insert($iter, "\n$indent");
 		$self->{htext}->scroll_mark_onscreen( $buffer->get_insert() );
 		return 1;
 	}
@@ -710,10 +1042,9 @@ makes it never reaching the widget.
 
 =cut
 
-sub parse_word {
+sub parse_word { # remember that $char can be empty
 	my ($self, $iter, $char) = @_;
-	return 0 unless $char eq ' ' or $char eq "\t";
-	return 0 if $self->_is_verbatim;
+	return 0 if $self->_is_verbatim($iter);
 	my $buffer = $self->{buffer};
 	my $lf = $iter->copy;
 	$self->{htext}->backward_display_line_start($lf)
@@ -730,6 +1061,17 @@ sub parse_word {
 		$buffer->insert($lf, $line);
 		return 1;
 	}
+	elsif ($line =~ /(?<!\S)\\(\w+)(\s*)$/) { # utf8 chars
+		my ($word, $post) = ($1, $2);
+		my $char = ($word =~ /^\d+$/) ? chr($word) :
+			exists($entity2char{$word}) ? $entity2char{$word} : undef ;
+		return 0 unless defined $char;
+		my $begin = $iter->copy;
+		$begin->backward_chars(1 + length $word);
+		$buffer->delete($begin, $iter);
+		$buffer->insert($begin, $char.$post);
+		return 1;
+	}
 #	elsif ($line =~ /^(\t|  )/) { # pre
 #		# FIXME \s* => \t
 #		$iter->forward_char unless $iter->is_end; # FIXME somthing at end
@@ -743,9 +1085,7 @@ sub parse_word {
 }
 
 sub _is_verbatim {
-	my $self = shift;
-	my $buffer = $self->{buffer};
-	my $iter = $buffer->get_iter_at_mark( $buffer->get_insert );
+	my ($self, $iter) = @_;
 	for ($iter->get_tags) {
 		return 1 if lc($_->get_property('name')) eq 'verbatim';
 	}
@@ -774,15 +1114,19 @@ Applies the tag with the name TAG to any selected text.
 
 sub apply_tag {
 	my ($self, $tag) = @_;
-	my ($start, $end) = $self->get_selection;
+	my $buffer = $self->{buffer};
+	my ($start, $end) = $buffer->auto_selection($tag);
 	return unless defined $start;
 
-	if ($start->get_line != $end->get_line) {
-		if ($tag eq 'verbatim') { $tag = 'Verbatim' }
-		# TODO what if selection contains linebreaks ??
+	if ($tag eq 'verbatim') {
+		my $Verbatim = $buffer->get_tag_table->lookup('Verbatim');
+		$tag = 'Verbatim'
+			if $start->get_line != $end->get_line
+			or $start->starts_line && $end->ends_line
+			or $start->has_tag($Verbatim)
+			or $end->has_tag($Verbatim)  ;
 	}
 	
-	my $buffer = $self->{buffer};
 	$buffer->remove_all_tags($start, $end);
 	$buffer->apply_tag_by_name($tag, $start, $end)
 		unless $tag eq 'normal';
@@ -796,23 +1140,6 @@ sub apply_tag {
 	}
 }
 
-=item C<get_selection(TAG)>
-
-=cut
-
-sub get_selection { # selects current word if no selections
-	my ($self, $tag) = @_;
-	my $buffer = $self->{buffer};
-	my ($start, $end) = $buffer->get_selection_bounds;
-#	print "selection: $start, $end\n";
-	if (!$end or $start == $end) {
-		# TODO autoselect word if defined tag
-		# ( or line if tag =~ /^head/ );
-		return undef;
-	}
-	return ($start, $end);
-}
-
 =item C<apply_link(LINK)>
 
 This method is called by the "Link" button or by the ^L keybinding.
@@ -830,15 +1157,15 @@ followed immediatly, but no actual link is made
 
 sub apply_link {
 	my ($self, $link) = @_;
-	my ($start, $end) = $self->get_selection('link');
+	my $buffer = $self->{buffer};
+	my ($start, $end) = $buffer->auto_selection('link');
 	
 	unless (defined $start) {
-		$self->{app}{settings}{read_only}
+		return $self->{app}{settings}{read_only}
 			? $self->{app}->goto_page_dialog
 			: $self->edit_link_dialog ;
 	}
 
-	my $buffer = $self->{buffer};
 	my $text = $buffer->get_text($start, $end, 0);
 	$link = $text unless defined $link;
 	return undef if $link =~ /\n/;
@@ -864,8 +1191,9 @@ bullets.
 
 sub toggle_bullets {
 	my ($self, $start, $end) = @_;
-	($start, $end) = $self->get_selection unless defined $start;
-	return unless defined $start;
+	my $buffer = $self->{buffer};
+	($start, $end) = $buffer->get_selection_bounds unless defined $start;
+	return if !$end or $start == $end;
 	
 	my $text = $self->{buffer}->get_text($start, $end, 1);
 	if ($text =~ /^\s*[\*\x{2022}]\s+/m) { # remove bullets
@@ -875,28 +1203,7 @@ sub toggle_bullets {
 		$text =~ s/^(\s*)(\S)/$1\x{2022} $2/mg;
 	}
 
-	$self->replace_selection($text, $start, $end);
-}
-
-=item C<replace_selection(TEXT, START, END)>
-
-=cut
-
-sub replace_selection {
-	my ($self, $text, $start, $end) = @_;
-	($start, $end) = $self->get_selection unless defined $start;
-	
-	my $buffer = $self->{buffer};
-	my $_start = $start->get_offset;
-	my $_end   = $start->get_offset + length $text;
-	
-	$buffer->delete($start, $end);
-	$buffer->insert($start, $text);
-
-	($start, $end) = map $buffer->get_iter_at_offset($_), ($_start, $_end);
-	$self->set_selection($start, $end);
-
-	return $start, $end;
+	$buffer->replace_selection($text);
 }
 
 =item C<selection_tab()>
@@ -907,15 +1214,15 @@ Puts a tab before every line of a selection.
 
 sub selection_tab {
 	my ($self, $start, $end) = @_;
-	($start, $end) = $self->get_selection unless defined $start;
-	return unless defined $start;
+	($start, $end) = $self->{buffer}->get_selection_bounds unless defined $start;
+	return if !$end or $start == $end;
 	
 	my $buffer = $self->{buffer};
 	my $text = $buffer->get_text($start, $end, 1);
 	$text =~ s/^/\t/mg;
 
-	my $verbatim = $self->_is_verbatim;
-	($start, $end) = $self->replace_selection($text, $start, $end);
+	my $verbatim = $self->_is_verbatim($start);
+	($start, $end) = $buffer->replace_selection($text);
 	$self->{buffer}->apply_tag_by_name('verbatim', $start, $end)
 		if $verbatim;
 }
@@ -928,13 +1235,14 @@ Removes a tab for every line of a selection.
 
 sub selection_backspace {
 	my ($self, $start, $end) = @_;
-	($start, $end) = $self->get_selection unless defined $start;
-	return unless defined $start;
+	my $buffer = $self->{buffer};
+	($start, $end) = $buffer->get_selection_bounds unless defined $start;
+	return if !$end or $start == $end;
 
 	my $text = $self->{buffer}->get_text($start, $end, 1);
-	my $verbatim = $self->_is_verbatim;
+	my $verbatim = $self->_is_verbatim($start);
 	if ($text =~ s/^\t//mg) {
-		($start, $end) = $self->replace_selection($text, $start, $end);
+		($start, $end) = $buffer->replace_selection($text);
 		$self->{buffer}->apply_tag_by_name('Verbatim', $start, $end)
 			if $verbatim;
 		return 1;
@@ -1040,7 +1348,7 @@ sub _do_step {
 	elsif ($action eq 'apply_tag') {
 		my $end = $buffer->get_iter_at_offset( $data[0] );
 		$buffer->remove_all_tags($start, $end);
-		if ($data[1] eq 'L') {
+		if ($data[1] eq 'link') {
 			$self->{htext}->apply_link($data[2], $start, $end);
 		}
 		else { $buffer->apply_tag_by_name($data[1], $start, $end) }
@@ -1062,16 +1370,19 @@ sub _do_step {
 #		
 #}
 
-=item C<edit_link_dialog()>
+=item C<edit_link_dialog(ITER)>
 
 This dialog allows the user to create a link for which
 the link target and the link text differ.
 
+If ITER is undefined the cursor is used.
+
 =cut
 
 sub edit_link_dialog {
-	my $self = shift;
-	my ($start, $end) = $self->get_selection('L');
+	my ($self, $iter) = @_;
+	my $buffer = $self->{buffer};
+	my ($start, $end) = $buffer->auto_selection('link', $iter);
 
 	my ($text, $link) = ('', '');
 	if (defined $start) {
@@ -1092,7 +1403,6 @@ sub edit_link_dialog {
 	$text = $link unless $text =~ /\S/;
 
 	# use delete + insert instead of apply because the text can be different
-	my $buffer = $self->{buffer};
 	if (defined $start) {
 		$buffer->delete($start, $end);
 	}
@@ -1100,10 +1410,23 @@ sub edit_link_dialog {
 		$start = $buffer->get_iter_at_mark( $buffer->get_insert());
 	}
 	my $bit = $link eq $text;
-	$self->{htext}->insert_link_at_iter($start, $text, [$bit, $link]);
-	$self->{buffer}->set_modified(1);
+	my $_start = $start->get_offset;
+	$buffer->insert($start, $text);
+	$start = $buffer->get_iter_at_offset($_start);
+	$end = $start->copy;
+	$end->forward_chars(length $text);
+	$buffer->remove_all_tags($start, $end); # because of this we can't use htext->insert_link()
+	$self->{htext}->apply_link([$bit, $link], $start, $end);
+	# FIXME redundant code from "apply_link" here
 }
 
+=item C<edit_image_dialog()>
+
+=cut
+
+sub edit_image_dialog {
+	# TODO 
+}
 
 1;
 
