@@ -5,7 +5,7 @@ use vars '$AUTOLOAD';
 use Gtk2;
 use Gtk2::Ex::PathBar; # custom widget
 
-our $VERSION = 0.01;
+our $VERSION = '0.10';
 
 =head1 NAME
 
@@ -23,6 +23,14 @@ Undefined methods are AUTOLOADED to the Gtk2::Ex::PathBar object.
 
 =over 4
 
+=item C<new(app => PARENT)>
+
+Simple constructor.
+
+=item C<init()>
+
+Method called by the constructor.
+
 =cut
 
 sub new {
@@ -59,7 +67,8 @@ sub widget { return $_[0]->{path_bar} }
 
 =item C<set_type(TYPE)>
 
-Set the type of the pathbar. Type can be 'namespace' or 'trace'.
+Set the type of the pathbar.
+Type can be 'history', 'recent' or 'namespace'.
 
 =cut
 
@@ -69,71 +78,98 @@ sub set_type {
 	$_[0]->update;
 }
 
+=item C<update()>
+
+Update the path displayed. Should be called when the current page changes.
+
+=cut
+
 sub update {
 	my $self = pop;
 	return unless $self->{app}{page};
 	if ($self->{type} eq 'namespace') {
-		$self->load_namespace;
+		$self->_load_namespace;
 	}
-	elsif ($self->{type} eq 'trace') {
-		$self->load_trace;
+	elsif ($self->{type} eq 'recent') {
+		$self->_load_recent;
+	}
+	elsif ($self->{type} eq 'history') {
+		$self->_load_history;
 	}
 	# else hidden, ignoring
 }
 
-sub load_namespace {
+sub _load_namespace {
 	my $self = shift;
-	my @path = grep {length $_} split /:+/, $self->{app}{page}->name();
-	my $path = ':'.join(':', @path);
+	my $namespace = $self->{app}->History->get_namespace();
+	$namespace =~ s/^:+|:+$//g;
+	my @namespace = split /:+/, $namespace;
+	my $current = $self->{app}->{page}->name;
+	$current =~ s/^:+|:+$//g;
+	my @current = split /:+/, $current;
 	
-	if ($self->{path_loaded} =~ /^$path/i) {
-		$self->{path_bar}->select_item($#path);
+	if ($self->{path_loaded} =~ /^$namespace/i) {
+		$self->{path_bar}->select_item($#current);
 		return;
 	}
 	
-	$self->{path_bar}->set_path(@path);
-	$self->{path_loaded} = $path;
+	$self->{path_loaded} = $namespace;
+	$self->{path_bar}->set_path(@namespace);
+	$self->{path_bar}->select_item($#current);
 	
 	for ($self->{path_bar}->get_items()) {
-		my $real = ':'.join(':', @{$_->{path_data}}) ;
+		my $name = ':'.join(':', @{$_->{path_data}}) ;
 		$_->signal_connect(enter => sub {
-			$self->{app}->push_status("Go to $real", 'link') } );
+			$self->{app}->push_status("Go to $name", 'link') } );
 		$_->signal_connect(leave => sub {
 			$self->{app}->pop_status('link')                 } );
 	}
 
 }
 
-sub load_trace {
+sub _load_history {
 	my $self = shift;
 
-	my ($back, $forw) = $self->{app}->History->state;
-	if ($self->{path_loaded} and $forw) {
-		$self->{path_bar}->select_item($back); # == $#back + 1
-		return;
-	}
+	#my $state = $self->{app}->History->get_state;
+	#if ($self->{path_loaded} and $$state{forw}) {
+	#	$self->{path_bar}->select_item($$state{back});
+	#	return;
+	#}
 
-	my ($hist, $page) = ($self->{app}->History, $self->{app}{page});
-	my (@path, @real);
-	for ($hist->get_back) {
+	my ($i, @hist) = $self->{app}->History->get_history;
+	my (@path, @names);
+	for (@hist) {
 		push @path, [$_->{basename}];
-		push @real, $_->{realname};
-	}
-	push @path, [$page->basename];
-	push @real, $page->realname;
-	for ($hist->get_forw) {
-		push @path, [$_->{basename}];
-		push @real, $_->{realname};
+		push @names, $_->{name};
 	}
+	#use Data::Dumper; print "point: $i\nlist: ", Dumper \@names;
 
 	$self->{path_bar}->set_path(@path);
-	$self->{path_bar}->select_item($back); # == $#back + 1
+	$self->{path_bar}->select_item($i);
 	$self->{path_loaded} = 1;
 
 	for ($self->{path_bar}->get_items) {
-		my $real = shift @real;
+		my $name = shift @names;
 		$_->signal_connect(enter => sub {
-			$self->{app}->push_status("Go to $real", 'link') } );
+			$self->{app}->push_status("Go to $name", 'link') } );
+		$_->signal_connect(leave => sub {
+			$self->{app}->pop_status('link')                 } );
+	}
+}
+
+sub _load_recent {
+	my $self = shift;
+	
+	my ($i, @recent) = $self->{app}->History->get_recent;
+	my @path = map [$_->{basename}, $_->{name}], @recent;
+
+	$self->{path_bar}->set_path(@path);
+	$self->{path_bar}->select_item($i);
+
+	for ($self->{path_bar}->get_items) {
+		my (undef, $name) = @{shift @path};
+		$_->signal_connect(enter => sub {
+			$self->{app}->push_status("Go to $name", 'link') } );
 		$_->signal_connect(leave => sub {
 			$self->{app}->pop_status('link')                 } );
 	}
@@ -144,13 +180,17 @@ sub on_path_clicked {
 	if ($self->{type} eq 'namespace') {
 		$self->{app}->load_page(join ':', @$path);
 	}
-	elsif ($self->{type} eq 'trace') {
-		my ($back, $forw) = $self->{app}->History->state;
+	elsif ($self->{type} eq 'history') {
+		my $back = $self->{app}->History->get_state->{back};
 		$idx -= $back;
 		if    ($idx < 0) { $self->{app}->go_back(-$idx) }
 		elsif ($idx > 0) { $self->{app}->go_forw($idx)  }
 		else             { $self->{app}->reload()       }
 	}
+	elsif ($self->{type} eq 'recent') {
+		my ($name) = @$path;
+		$self->{app}->load_page($name);
+	}
 	# else bug !?
 }
 
