@@ -5,7 +5,7 @@ use vars '$AUTOLOAD';
 use Gtk2;
 use Gtk2::Gdk::Keysyms;
 
-our $VERSION = '0.10';
+our $VERSION = '0.12';
 
 =head1 NAME
 
@@ -43,7 +43,6 @@ sub new {
 
 sub init { # called by new()
 	my $self = shift;
-	$self->{tree_iter} = {};
 
 	my $scroll_window = Gtk2::ScrolledWindow->new();
 	$scroll_window->set_policy('automatic', 'automatic');
@@ -51,7 +50,7 @@ sub init { # called by new()
 	$self->{scroll_window} = $scroll_window;
 
 	my $tree_store = Gtk2::TreeStore->new('Glib::String', 'Glib::String');
-	#$tree_store->set_sort_column_id(0, 'ascending');
+#	$tree_store->signal_connect(row_changed => \&on_row_changed, $self);
 	$self->{tree_store} = $tree_store;
 
 	my $tree_view = Gtk2::TreeView->new($tree_store);
@@ -62,7 +61,7 @@ sub init { # called by new()
         	'Pages', $renderer, 'text', 0);
 	$tree_view->append_column($page_column);
 	$tree_view->set_headers_visible(0);
-	#$tree_view->set_reorderable(1);
+#	$tree_view->set_reorderable(1);
 	$tree_view->get_selection->set_mode('browse'); # one item selected at all times
 	$tree_view->signal_connect(key_press_event      => \&on_key_press_event);
 	$tree_view->signal_connect(button_release_event => \&on_button_release_event, $self);
@@ -74,16 +73,6 @@ sub init { # called by new()
 	$scroll_window->add($tree_view);
 	$self->{tree_view} = $tree_view;
 
-#	my @buttons;
-#	push @buttons, $self->{app}->add_button(
-#        	'Expand All', 'Ctrl +', 'gtk-zoom-in', sub {$tree_view->expand_all});
-#	$self->{app}->add_key('Ctrl KP_Add', sub {$tree_view->expand_all});
-#	$self->{app}->add_key('Ctrl =', sub {$tree_view->expand_all});
-#	push @buttons, $self->{app}->add_button(
-#		'Collapse All', 'Ctrl -', 'gtk-zoom-out', sub {$tree_view->collapse_all});
-#	$self->{app}->add_key('Ctrl KP_Subtract', sub {$tree_view->collapse_all});
-#	$self->{buttons} = \@buttons;
-	
 	$self->{app}->signal_connect(page_loaded => sub {$self->select_page(pop)});
 	$self->{app}->signal_connect(page_deleted => sub {$self->unlist_page(pop)});
 	$self->{app}->signal_connect(page_created => sub {
@@ -109,30 +98,6 @@ Also use this widget for things like show_all() and hide_all().
 
 sub widget { return $_[0]->{scroll_window} }
 
-=item C<hide()>
-
-Hides the root widget and all asociated toolbar buttons.
-
-=cut
-
-sub hide { # FIXME belongs in base class
-	$_[0]->widget->hide;
-	$_->hide for @{$_[0]->{buttons}};
-	#$_[0]->{app}{toolbar}->queue_draw;
-}
-
-=item C<show()>
-
-Shows the root widget and all asociated toolbar buttons.
-
-=cut
-
-sub show { # FIXME belongs in base class
-	$_[0]->widget->show;
-	$_->show for @{$_[0]->{buttons}};
-	#$_[0]->{app}{toolbar}->queue_draw;
-}
-
 sub on_key_press_event { # some extra keybindings for the treeview
 	my ($tree_view, $event) = @_;
 	my $val = $event->keyval();
@@ -168,23 +133,57 @@ sub on_button_release_event {
 sub on_row_activated { # load a page when clicked or entered
 	my $self = pop;
 	my ($tree_view, $path) = @_;
-	my $page = $self->page_from_path($path);
+	my $page = $self->path2page($path);
 	$self->{app}->load_page($page)
 		unless $page eq $self->{app}{page}->name;
 }
 
-=item C<page_from_path(PATH)>
+sub on_row_changed { # Used to control drag-n-drop
+	my ($model, $path, $iter, $self) = @_;
+	my $parent = $model->iter_parent($iter);
+	$parent = $parent ? $model->get($parent, 1) : '' ;
+	my ($base, $page) = $model->get($iter);
+	return if $page eq $parent.':'.$base;
+	my $ok = $self->{app}->rename_page($page => $parent.':'.$base);
+	return if $ok;
+
+	# TODO put data back if move failed
+	
+}
+
+=item C<page2path(PAGE)>
+
+Returns the TreePath corresponding to PAGE.
+
+=cut
+
+sub page2path {
+	my ($self, $page) = @_;
+	$page =~ s/^:+|:+$//g;
+	my @page = split /:+/, $page;
+	my $model = $self->{tree_view}->get_model;
+	my $iter;
+	for my $p (@page) {
+		$iter = $model->iter_children($iter) || return;
+			# returns root iter when iter is undef
+		while ($model->get($iter, 0) ne $p) {
+			$iter = $model->iter_next($iter) || return;
+		}
+	}
+	return $model->get_path($iter);
+}
+
+=item C<path2page(PATH)>
 
 Returns the page name corresponding to a certain TreePath.
 
 =cut
 
-sub page_from_path {
+sub path2page {
 	my ($self, $path) = @_;
 	my $model = $self->{tree_view}->get_model;
 	my $iter = $model->get_iter($path);
-	my ($basename, $name) = $model->get($iter);
-	return wantarray ? ($name, $basename) : $name;
+	return $model->get($iter, 1);
 }
 
 =item C<activate_if_selected(PATH, COLUMN)>
@@ -203,7 +202,7 @@ sub activate_if_selected { # single-click navigation
 	my ($selected) = $tree_view->get_selection->get_selected_rows;
 	return 0 unless $selected and 0 == ($path->compare($selected));
 
-	my $page = $self->page_from_path($path);
+	my $page = $self->path2page($path);
 	$column ||= $tree_view->get_column(0);
 	$tree_view->row_activated($path, $column)
 		unless $page eq $self->{app}{page}->name;
@@ -220,12 +219,12 @@ EVENT is an optional argument.
 sub popup_context_menu {
 	my ($self, $path, $event) = @_;
 	my $tree_view = $self->{tree_view};
-	my $page = $self->page_from_path($path);
+	my $page = $self->path2page($path);
 	
 	my $menu = Gtk2::Menu->new();
-	my $item1 = Gtk2::MenuItem->new('Move page');
+	my $item1 = Gtk2::MenuItem->new('Rename page');
 	$item1->signal_connect(
-		activate => sub { $self->{app}->move_page($page, undef, 1) });
+		activate => sub { $self->{app}->rename_page($page) });
 	$menu->append($item1);
 	my $item2 = Gtk2::MenuItem->new('Delete page');
 	$item2->signal_connect(
@@ -250,12 +249,12 @@ Will be DEPRECATED.
 
 =cut
 
-sub load_index {
+sub load_index { # TODO remove this from Zim.pm and PageView.pm
 	my $self = shift;
 	return if $self->{_loaded};
 	$self->{tree_store}->clear; # just to be sure
-	$self->{tree_iter} = {};
 	eval { $self->list_pages(':') };
+	warn $@ if $@;
 	$self->select_page($self->{app}{page}->name);
 	$self->{app}->message("Index loaded");
 	$self->{_loaded} = 1;
@@ -263,20 +262,26 @@ sub load_index {
 
 =item C<list_pages(NAMESPACE)>
 
-Wrapper around C<Zim->list_pages> that fills the index tree.
+Wrapper around C<< Zim::Repository->list_pages() >> to fill the tree.
 
 =cut
 
 sub list_pages {
-	my ($self, $namespace) = @_;
-	#print "> $namespace\n";
-	for ($self->{app}{repository}->list_pages($namespace)) {
-		#print "\t$_\n";
-		my $is_dir = ($_ =~ s/:+$//);
-		$self->list_page($namespace.$_);
-		$self->list_pages($namespace.$_.':') if $is_dir; # recurs
+	my ($self, $namespace, $iter) = @_; # last argument is private
+	#warn "list_pages: $namespace\n";
+	$self->{tree_store}->signal_handlers_block_by_func(\&on_row_changed);
+	$iter ||= $self->list_page($namespace);
+	$namespace =~ s/:?$/:/;
+	my $model = $self->{tree_view}->get_model;
+	# TODO check for existing children
+	for my $p ($self->{app}{repository}->list_pages($namespace)) {
+		#warn "\t$p\n";
+		my $is_dir = ($p =~ s/:+$//);
+		my $child = $model->append($iter);
+		$model->set($child, 0 => $p, 1 => $namespace.$p); # base_name, name
+		$self->list_pages($namespace.$p, $child) if $is_dir; # recurse
 	}
-	Gtk2->main_iteration while (Gtk2->events_pending);
+	$self->{tree_store}->signal_handlers_unblock_by_func(\&on_row_changed);
 }
 
 =item C<list_page(REALNAME)>
@@ -286,26 +291,28 @@ Adds a page to the index tree.
 =cut
 
 sub list_page {
-	my ($self, $name) = @_;
-	return unless defined $name;
-	
-	my ($tree_store, $tree_iter) = @$self{qw/tree_store tree_iter/};
-	$name =~ s/^:+|:+$//g;
-	my ($page, @parts) = (undef, split /:+/, $name);
+	my ($self, $page) = @_;
+	$page =~ s/^:+|:+$//g;
+	my @page = split /:+/, $page;
+	$page = '';
+	my $model = $self->{tree_view}->get_model;
+	$model->signal_handlers_block_by_func(\&on_row_changed);
 	my $iter;
-	while (@parts) {
-		$page = $page.':'.$parts[0];
-		unless (exists $$tree_iter{$page}) {
-			$$tree_iter{$page} = $tree_store->append($iter);
-			$tree_store->set_value($$tree_iter{$page},
-				0 => $parts[0],
-				1 => $page      );
+	for my $p (@page) {
+		$page .= ':'.$p;
+		my $child = $model->iter_children($iter);
+			# returns root iter when iter is undef
+		while ($child and $model->get($child, 0) ne $p) {
+			$child = $model->iter_next($child);
 		}
-		$iter = $$tree_iter{$page};
-		shift @parts;
+		unless ($child) {
+			$child = $model->append($iter);
+			$model->set($child, 0 => $p, 1 => $page); # base_name, name
+		}
+		$iter = $child;
 	}
-
-	return 1;
+	$model->signal_handlers_unblock_by_func(\&on_row_changed);
+	return $iter;
 }
 
 =item C<unlist_page(REALNAME)>
@@ -316,35 +323,27 @@ Removes a page from the index unless it has children.
 
 sub unlist_page {
 	my ($self, $page) = @_;
-	my ($tree_store, $tree_iter) = @$self{qw/tree_store tree_iter/};
-	return unless defined $page;
-	$page =~ s/::+/:/g;
-	$page =~ s/:$//;
-	return unless exists $$tree_iter{$page};
-	return if $tree_store->iter_children($$tree_iter{$page}); # has children
-	$tree_store->remove($$tree_iter{$page});
-	delete $$tree_iter{$page};
+	my $path = $self->page2path($page) || return;
+	my $model = $self->{tree_view}->get_model;
+	my $iter = $model->get_iter($path);
+	return if $model->iter_children($iter); # has children
+	$model->remove($iter);
 }
 
 =item C<select_page(PAGE)>
 
-Update the TreeView to display and select a certain page.
+Update the TreeView to display and highlight a certain page.
 
 =cut
 
 sub select_page {
 	my ($self, $page) = @_;
-	my ($tree_view, $tree_store, $tree_iter) = @$self{qw/tree_view tree_store tree_iter/};
-	$page =~ s/::+/:/g;
-	$page =~ s/:$//;
-	return unless exists $$tree_iter{$page};
-	
-	my $path = $tree_store->get_path($$tree_iter{$page});
-	$tree_view->expand_to_path($path);
-	$tree_view->get_selection->select_path($path);
-
-	$tree_view->set_cursor($path);
-	$tree_view->scroll_to_cell($path, undef, 0); # vert. scroll
+	my $path = $self->page2path($page) || return;
+	my $view = $self->{tree_view};
+	$view->expand_to_path($path);
+	$view->get_selection->select_path($path);
+	$view->set_cursor($path);
+	$view->scroll_to_cell($path, undef, 0); # vert. scroll
 }
 
 1;
