@@ -2,7 +2,7 @@ package Zim::Page;
 
 use strict;
 
-our $VERSION = '0.10';
+our $VERSION = '0.12';
 
 =head1 NAME
 
@@ -42,11 +42,31 @@ sub new {
 		name => $name,
 		repository => $parent,
 		status => '',
-		properties => {},
+		properties => {read_only => 1},
 	}, $class ;
+	$self->init();
 	return $self;
 }
 
+=item C<init()>
+
+Called by C<new()>. To be overloaded.
+
+=cut
+
+sub init { }
+
+=item C<has_interface(NAME)>
+
+Checks whether this object supports a certain interface.
+
+=cut
+
+sub has_interface {
+	my ($self, $name) = @_;
+	return scalar grep {$_ eq $name} $self->interfaces;
+}
+
 =item C<interfaces()>
 
 Returns a list of interfaces supported by this page.
@@ -114,27 +134,53 @@ sub status {
 	return $_[0]->{status};
 }
 
-=item C<close()>
+=item C<exists()>
 
-=item C<delete()>
-
-=item C<move(NEW_NAME)>
-
-=item C<resolve_link(LINK)>
-
-The methods C<close()>, C<delete()>, C<move()> and C<resolve_link()>
-are aliases for the methods C<close_page()>, C<delete_page()>, C<move_page()>
-and C<resolve_link()> respectively in L<Zim::Repository>.
+Returns TRUE if the page already exists.
 
 =cut
 
-sub close { $_[0]->{repository}->close_page(@_) }
+sub exists { $_[0]->{status} ne 'new' and $_[0]->{status} ne 'deleted' }
+
+=item C<copy(TARGET)>
+
+=item C<move(TARGET)>
+
+=item C<delete()>
+
+The methods C<copy()>, C<move()> and C<delete()> are aliases for the methods
+C<copy_page()>, C<move_page()> and C<delete_page()> in the public repository
+interface; see L<Zim::Repository>.
+
+=cut
+
+sub copy { $_[0]->{repository}->root->copy_page(@_) }
+
+sub move { $_[0]->{repository}->root->move_page(@_) }
 
 sub delete { $_[0]->{repository}->delete_page(@_) }
 
-sub move { $_[0]->{repository}->move_page(@_) }
+=item C<resolve_link(LINK)>
 
-sub resolve_link { $_[0]->{repository}->resolve_link(@_) }
+Returns a page object for LINK where LINK can be resolved
+relative to the current page.
+
+=cut
+
+sub resolve_link { # 'cloning' hack needed when exporting
+	my $obj = $_[0]->{cloning} ? $_[0]->{cloning} : $_[0];
+	$obj->{repository}->resolve_link(@_)
+}
+
+=item C<clone(SOURCE)>
+
+Import content from object SOURCE into this object.
+This should be implemented depending on the interfaces supported
+by both SOURCE and TARGET.
+
+=cut
+
+sub clone { die "Can not copy object of class ".ref($_[0])."\n" }
 
 1;
 
@@ -172,6 +218,7 @@ Currently only the file:// url is really supported.
 =item read_only (boolean)
 
 Tells the interface that this page should not be edited.
+Defaults to TRUE.
 
 =back
 
